@@ -59,10 +59,10 @@ public:
     public:
         typedef id_type size_type;
         
-        bm_func_base(blocks_manager& bman) : bm_(bman) {}
+        bm_func_base(blocks_manager& bman) BMNOEXCEPT : bm_(bman) {}
 
-        void on_empty_top(unsigned /* top_block_idx*/ ) {}
-        void on_empty_block(block_idx_type /* block_idx*/ ) {}
+        void on_empty_top(unsigned /* top_block_idx*/ ) BMNOEXCEPT {}
+        void on_empty_block(block_idx_type /* block_idx*/ )BMNOEXCEPT {}
     private:
         bm_func_base(const bm_func_base&);
         bm_func_base& operator=(const bm_func_base&);
@@ -76,13 +76,13 @@ public:
     {
     public:
         typedef id_type size_type;
-        bm_func_base_const(const blocks_manager& bman) : bm_(bman) {}
+        bm_func_base_const(const blocks_manager& bman) BMNOEXCEPT : bm_(bman) {}
 
-        void on_empty_top(unsigned /* top_block_idx*/ ) {}
-        void on_empty_block(block_idx_type /* block_idx*/ ) {}
+        void on_empty_top(unsigned /* top_block_idx*/ ) BMNOEXCEPT {}
+        void on_empty_block(block_idx_type /* block_idx*/ ) BMNOEXCEPT {}
     private:
-        bm_func_base_const(const bm_func_base_const&);
-        bm_func_base_const& operator=(const bm_func_base_const&);
+        bm_func_base_const(const bm_func_base_const&) BMNOEXCEPT;
+        bm_func_base_const& operator=(const bm_func_base_const&) BMNOEXCEPT;
     protected:
         const blocks_manager&  bm_;
     };
@@ -92,10 +92,10 @@ public:
     class block_count_base : public bm_func_base_const
     {
     protected:
-        block_count_base(const blocks_manager& bm) 
+        block_count_base(const blocks_manager& bm) BMNOEXCEPT
             : bm_func_base_const(bm) {}
 
-        bm::id_t block_count(const bm::word_t* block) const
+        bm::id_t block_count(const bm::word_t* block) const BMNOEXCEPT
         {
             return this->bm_.block_bitcount(block);
         }
@@ -108,17 +108,17 @@ public:
     public:
         typedef id_type size_type;
 
-        block_count_func(const blocks_manager& bm) 
+        block_count_func(const blocks_manager& bm) BMNOEXCEPT
             : block_count_base(bm), count_(0) {}
 
-        id_type count() const { return count_; }
+        id_type count() const BMNOEXCEPT { return count_; }
 
-        void operator()(const bm::word_t* block)
+        void operator()(const bm::word_t* block) BMNOEXCEPT
         {
             count_ += this->block_count(block);
         }
-        void add_full(id_type c) { count_ += c; }
-        void reset() { count_ = 0; }
+        void add_full(id_type c) BMNOEXCEPT { count_ += c; }
+        void reset() BMNOEXCEPT { count_ = 0; }
 
     private:
         id_type count_;
@@ -131,24 +131,22 @@ public:
     public:
         typedef id_type size_type;
 
-        block_count_arr_func(const blocks_manager& bm, unsigned* arr) 
+        block_count_arr_func(const blocks_manager& bm, unsigned* arr) BMNOEXCEPT
             : block_count_base(bm), arr_(arr), last_idx_(0) 
         {
             arr_[0] = 0;
         }
 
-        void operator()(const bm::word_t* block, id_type idx)
+        void operator()(const bm::word_t* block, id_type idx) BMNOEXCEPT
         {
             while (++last_idx_ < idx)
-            {
                 arr_[last_idx_] = 0;
-            }
             arr_[idx] = this->block_count(block);
             last_idx_ = idx;
         }
 
-        id_type last_block() const { return last_idx_; }
-        void on_non_empty_top(unsigned) {}
+        id_type last_block() const BMNOEXCEPT { return last_idx_; }
+        void on_non_empty_top(unsigned) BMNOEXCEPT {}
 
     private:
         unsigned*  arr_;
@@ -161,13 +159,14 @@ public:
     public:
         typedef id_type size_type;
 
-        block_count_change_func(const blocks_manager& bm) 
+        block_count_change_func(const blocks_manager& bm) BMNOEXCEPT
             : bm_func_base_const(bm),
                 count_(0),
                 prev_block_border_bit_(0)
         {}
 
-        block_idx_type block_count(const bm::word_t* block, block_idx_type idx)
+        block_idx_type block_count(const bm::word_t* block,
+                                   block_idx_type idx) BMNOEXCEPT
         {
             block_idx_type cnt = 0;
             id_type first_bit;
@@ -187,7 +186,7 @@ public:
                 if (BM_IS_GAP(block))
                 {
                     gap_word_t* gap_block = BMGAP_PTR(block);
-                    cnt = gap_length(gap_block) - 1;
+                    cnt = bm::gap_length(gap_block) - 1;
                     if (idx)
                     {
                         first_bit = bm::gap_test_unr(gap_block, 0);
@@ -213,9 +212,9 @@ public:
             return cnt;
         }
         
-        id_type count() const { return count_; }
+        id_type count() const BMNOEXCEPT { return count_; }
 
-        void operator()(const bm::word_t* block, block_idx_type idx)
+        void operator()(const bm::word_t* block, block_idx_type idx) BMNOEXCEPT
         {
             count_ += block_count(block, idx);
         }
@@ -232,11 +231,12 @@ public:
     public:
         typedef id_type size_type;
 
-        block_any_func(const blocks_manager& bm) 
+        block_any_func(const blocks_manager& bm) BMNOEXCEPT
             : bm_func_base_const(bm) 
         {}
 
-        bool operator()(const bm::word_t* block, block_idx_type /*idx*/)
+        bool operator()
+                (const bm::word_t* block, block_idx_type /*idx*/) BMNOEXCEPT
         {
             if (BM_IS_GAP(block)) // gap block
                 return (!gap_is_all_zero(BMGAP_PTR(block)));
@@ -250,9 +250,9 @@ public:
     class gap_level_func : public bm_func_base
     {
     public:
-        gap_level_func(blocks_manager& bm, const gap_word_t* glevel_len)
-            : bm_func_base(bm),
-                glevel_len_(glevel_len)
+        gap_level_func(blocks_manager& bm,
+                       const gap_word_t* glevel_len) BMNOEXCEPT
+            : bm_func_base(bm), glevel_len_(glevel_len)
         {
             BM_ASSERT(glevel_len);
         }
@@ -282,19 +282,18 @@ public:
                 return;
             }
 
-            unsigned len = gap_length(gap_blk);
-            int level = gap_calc_level(len, glevel_len_);
+            unsigned len = bm::gap_length(gap_blk);
+            int level = bm::gap_calc_level(len, glevel_len_);
             if (level == -1)
             {
-                bm::word_t* blk = 
-                    bman.get_allocator().alloc_bit_block();
+                bm::word_t* blk = bman.get_allocator().alloc_bit_block();
                 bman.set_block_ptr(idx, blk);
                 bm::gap_convert_to_bitset(blk, gap_blk);
             }
             else
             {
                 gap_word_t* gap_blk_new = 
-                    bman.allocate_gap_block(unsigned(level), gap_blk, glevel_len_);
+                bman.allocate_gap_block(unsigned(level), gap_blk, glevel_len_);
 
                 bm::word_t* p = (bm::word_t*) gap_blk_new;
                 BMSET_PTRGAP(p);
@@ -312,7 +311,7 @@ public:
     class block_one_func : public bm_func_base
     {
     public:
-        block_one_func(blocks_manager& bm) : bm_func_base(bm) {}
+        block_one_func(blocks_manager& bm) BMNOEXCEPT : bm_func_base(bm) {}
 
         void operator()(bm::word_t* block, block_idx_type idx)
         {
@@ -357,7 +356,7 @@ public:
     }
     
 #ifndef BM_NO_CXX11
-    blocks_manager(blocks_manager&& blockman) BMNOEXEPT
+    blocks_manager(blocks_manager&& blockman) BMNOEXCEPT
         : max_bits_(blockman.max_bits_),
           top_blocks_(0),
           top_block_size_(blockman.top_block_size_),
@@ -369,7 +368,7 @@ public:
     }
 #endif
 
-    ~blocks_manager() BMNOEXEPT
+    ~blocks_manager() BMNOEXCEPT
     {
         if (temp_block_)
             alloc_.free_bit_block(temp_block_);
@@ -379,7 +378,7 @@ public:
     /*! \brief Swaps content 
         \param bm  another blocks manager
     */
-    void swap(blocks_manager& bm) BMNOEXEPT
+    void swap(blocks_manager& bm) BMNOEXCEPT
     {
         BM_ASSERT(this != &bm);
 
@@ -399,7 +398,7 @@ public:
     
     /*! \brief implementation of moving semantics
     */
-    void move_from(blocks_manager& bm) BMNOEXEPT
+    void move_from(blocks_manager& bm) BMNOEXCEPT
     {
         deinit_tree();
         swap(bm);
@@ -412,9 +411,9 @@ public:
     }
     
 
-    void free_ptr(bm::word_t** ptr)
+    void free_ptr(bm::word_t** ptr) BMNOEXCEPT
     {
-        if (ptr) alloc_.free_ptr(ptr);
+        alloc_.free_ptr(ptr);
     }
 
     /**
@@ -422,7 +421,7 @@ public:
         \param bits_to_store - supposed capacity (number of bits)
         \return size of the top level block
     */
-    unsigned compute_top_block_size(id_type bits_to_store)
+    unsigned compute_top_block_size(id_type bits_to_store) const BMNOEXCEPT
     {
         if (bits_to_store >= bm::id_max)  // working in full-range mode
             return bm::set_top_array_size;
@@ -446,6 +445,7 @@ public:
     }
     */
 
+
     /**
         \brief Finds block in 2-level blocks array  
         Specilized version of get_block(unsigned), returns an additional
@@ -455,7 +455,8 @@ public:
         \param no_more_blocks - 1 if there are no more blocks at all
         \return block adress or NULL if not yet allocated
     */
-    bm::word_t* get_block(block_idx_type nb, int* no_more_blocks) const
+    const bm::word_t*
+    get_block(block_idx_type nb, int* no_more_blocks) const BMNOEXCEPT
     {
         BM_ASSERT(top_blocks_);
         unsigned i = unsigned(nb >> bm::set_array_shift);
@@ -480,16 +481,6 @@ public:
         return ret;
     }
 
-    /** 
-    Recalculate absolute block address into coordinates
-    */
-    static
-    BMFORCEINLINE
-    void get_block_coord(block_idx_type nb, unsigned& i, unsigned& j)
-    {
-        i = unsigned(nb >> bm::set_array_shift); // top block address
-        j = unsigned(nb &  bm::set_array_mask);  // address in sub-block
-    }
 
     /**
     Find the next non-zero block starting from nb
@@ -498,7 +489,7 @@ public:
     @return bm::set_total_blocks - no more blocks
     */
     block_idx_type
-    find_next_nz_block(block_idx_type nb, bool deep_scan = true) const
+    find_next_nz_block(block_idx_type nb, bool deep_scan=true) const BMNOEXCEPT
     {
         if (is_init())
         {
@@ -530,7 +521,7 @@ public:
         \param j - second level block index
         \return block adress or NULL if not yet allocated
     */
-    const bm::word_t* get_block(unsigned i, unsigned j) const
+    const bm::word_t* get_block(unsigned i, unsigned j) const BMNOEXCEPT
     {
         if (!top_blocks_ || i >= top_block_size_) return 0;
         const bm::word_t* const* blk_blk = top_blocks_[i];
@@ -546,7 +537,7 @@ public:
         \param j - second level block index
         \return block adress or NULL if not yet allocated
     */
-    const bm::word_t* get_block_ptr(unsigned i, unsigned j) const
+    const bm::word_t* get_block_ptr(unsigned i, unsigned j) const BMNOEXCEPT
     {
         if (!top_blocks_ || i >= top_block_size_) return 0;
 
@@ -562,9 +553,10 @@ public:
         \param j - second level block index
         \return block adress or NULL if not yet allocated
     */
-    bm::word_t* get_block_ptr(unsigned i, unsigned j)
+    bm::word_t* get_block_ptr(unsigned i, unsigned j) BMNOEXCEPT
     {
-        if (!top_blocks_ || i >= top_block_size_) return 0;
+        if (!top_blocks_ || i >= top_block_size_)
+            return 0;
         bm::word_t* const* blk_blk = top_blocks_[i];
         if ((bm::word_t*)blk_blk == FULL_BLOCK_FAKE_ADDR)
             return FULL_BLOCK_FAKE_ADDR;
@@ -578,7 +570,7 @@ public:
         \param i - top level block index
         \return block adress or NULL if not yet allocated
     */
-    const bm::word_t* const * get_topblock(unsigned i) const
+    const bm::word_t* const * get_topblock(unsigned i) const BMNOEXCEPT
     {
         return (!top_blocks_ || i >= top_block_size_) ? 0 : top_blocks_[i];
     }
@@ -586,7 +578,7 @@ public:
     /** 
         \brief Returns root block in the tree.
     */
-    bm::word_t*** top_blocks_root() const
+    bm::word_t*** top_blocks_root() const BMNOEXCEPT
     {
         blocks_manager* bm = 
             const_cast<blocks_manager*>(this);
@@ -706,9 +698,18 @@ public:
         
         unsigned i, j, i_from, j_from, i_to, j_to;
         get_block_coord(nb, i_from, j_from);
+
+        if (i_from >= top_block_size_) // nothing to do
+            return;
+
         get_block_coord(nb_to, i_to, j_to);
-        
-        reserve_top_blocks(i_to+1); // TODO: why do it if it is zero anyway?        
+
+        if (i_to >= top_block_size_)
+        {
+            i_to = top_block_size_-1;
+            j_to = bm::set_sub_array_size;
+        }
+                
         bm::word_t*** blk_root = top_blocks_root();
         
         if (i_from == i_to)  // same subblock
@@ -716,8 +717,9 @@ public:
             bm::word_t** blk_blk = blk_root[i_from];
             if (blk_blk)
             {
+                j_to -= (j_to == bm::set_sub_array_size);
                 for (j = j_from; j <= j_to; ++j)
-                    zero_block(i_from, j);
+                    zero_block(i_from, j); // TODO: optimization (lots of ifs in this loop)
             }
             return;
         }
@@ -742,11 +744,13 @@ public:
             --i_to; // safe because (i_from == i_to) case is covered
         }
         // process all full sub-lanes
-        //
+        // TODO: loop unroll /SIMD
         for (i = i_from; i <= i_to; ++i)
         {
             bm::word_t** blk_blk = blk_root[i];
-            if (!blk_blk || blk_blk == (bm::word_t**)FULL_BLOCK_FAKE_ADDR)
+            if (!blk_blk)
+                continue;
+            if (blk_blk == (bm::word_t**)FULL_BLOCK_FAKE_ADDR)
             {
                 blk_root[i] = 0;
                 continue;
@@ -834,7 +838,7 @@ public:
         {
             gap_res = true;
             new_block = (bm::word_t*)
-                    get_allocator().alloc_gap_block(unsigned(new_level), glen());
+                get_allocator().alloc_gap_block(unsigned(new_level), glen());
             ::memcpy(new_block, gap_block, len * sizeof(bm::gap_word_t));
             bm::set_gap_level(new_block, new_level);
         }
@@ -1047,7 +1051,7 @@ public:
                                      bool     allow_null_ret=true)
     {
         unsigned i, j;
-        this->get_block_coord(nb, i, j);
+        bm::get_block_coord(nb, i, j);
         bm::word_t*  block = this->get_block_ptr(i, j);
 
         if (!IS_VALID_ADDR(block)) // NULL block or ALLSET
@@ -1091,7 +1095,7 @@ public:
     bm::word_t* check_allocate_block(block_idx_type nb, int initial_block_type)
     {
         unsigned i, j;
-        this->get_block_coord(nb, i, j);
+        bm::get_block_coord(nb, i, j);
         bm::word_t* block = this->get_block_ptr(i, j);
 
         if (!IS_VALID_ADDR(block)) // NULL block or ALLSET
@@ -1121,7 +1125,7 @@ public:
     /*! @brief Fills all blocks with 0.
         @param free_mem - if true function frees the resources (obsolete)
     */
-    void set_all_zero(bool /*free_mem*/)
+    void set_all_zero(bool /*free_mem*/) BMNOEXCEPT
     {
         if (!is_init()) return;
         deinit_tree(); // TODO: optimization of top-level realloc
@@ -1138,7 +1142,7 @@ public:
                                 bm::set_sub_array_size, func);
     }
     
-    void free_top_subblock(unsigned nblk_blk)
+    void free_top_subblock(unsigned nblk_blk) BMNOEXCEPT
     {
         BM_ASSERT(top_blocks_[nblk_blk]);
         if ((bm::word_t*)top_blocks_[nblk_blk] != FULL_BLOCK_FAKE_ADDR)
@@ -1374,7 +1378,17 @@ public:
         // non-compressable bit-block
         copy_bit_block(i, j, src_block);
     }
-    
+
+    /**
+        Optimize bit-block
+    */
+    void optimize_bit_block(block_idx_type nb)
+    {
+        unsigned i, j;
+        bm::get_block_coord(nb, i, j);
+        optimize_bit_block(i, j);
+    }
+
     /**
         Optimize bit-block at i-j position
     */
@@ -1439,7 +1453,7 @@ public:
         Places new block into blocks table.
     */
     BMFORCEINLINE
-    void set_block_ptr(unsigned i, unsigned j, bm::word_t* block)
+    void set_block_ptr(unsigned i, unsigned j, bm::word_t* block) BMNOEXCEPT
     {
         BM_ASSERT(is_init());
         BM_ASSERT(i < top_block_size_);
@@ -1513,7 +1527,7 @@ public:
             gap_word_t* gap_block = BMGAP_PTR(block);
             
             bm::word_t* new_block = alloc_.alloc_bit_block();
-            gap_convert_to_bitset(new_block, gap_block);
+            bm::gap_convert_to_bitset(new_block, gap_block);
             alloc_.free_gap_block(gap_block, this->glen());
             
             set_block_ptr(nb, new_block);
@@ -1584,7 +1598,7 @@ public:
     /**
     Free block, make it zero pointer in the tree
     */
-    void zero_gap_block_ptr(unsigned i, unsigned j)
+    void zero_gap_block_ptr(unsigned i, unsigned j) BMNOEXCEPT
     {
         BM_ASSERT(top_blocks_ && i < top_block_size_);
         
@@ -1603,19 +1617,15 @@ public:
         Count number of bits ON in the block
     */
     static
-    bm::id_t block_bitcount(const bm::word_t* block)
+    bm::id_t block_bitcount(const bm::word_t* block) BMNOEXCEPT
     {
         BM_ASSERT(block);
         id_t count;
         if (BM_IS_GAP(block))
-        {
             count = bm::gap_bit_count_unr(BMGAP_PTR(block));
-        }
         else // bitset
-        {
             count = (IS_FULL_BLOCK(block)) ? bm::bits_in_block
                                            : bm::bit_block_count(block);
-        }
         return count;
     }
 
@@ -1665,7 +1675,7 @@ public:
     }
     
     /*! deallocate temp block */
-    void free_temp_block()
+    void free_temp_block() BMNOEXCEPT
     {
         if (temp_block_)
         {
@@ -1673,6 +1683,7 @@ public:
             temp_block_ = 0;
         }
     }
+
     /*! Detach and return temp block.
         if temp block is NULL allocates a bit-block
         caller is responsible for returning
@@ -1692,7 +1703,7 @@ public:
     /*! Return temp block
         if temp block already exists - block gets deallocated
     */
-    void return_tempblock(bm::word_t* block)
+    void return_tempblock(bm::word_t* block) BMNOEXCEPT
     {
         BM_ASSERT(block != temp_block_);
         BM_ASSERT(IS_VALID_ADDR(block));
@@ -1704,7 +1715,7 @@ public:
     }
 
     /*! Assigns new GAP lengths vector */
-    void set_glen(const gap_word_t* glevel_len)
+    void set_glen(const gap_word_t* glevel_len) BMNOEXCEPT
     {
         ::memcpy(glevel_len_, glevel_len, sizeof(glevel_len_));
     }
@@ -1732,7 +1743,7 @@ public:
     
     /** Returns true if second level block pointer is 0.
     */
-    bool is_subblock_null(unsigned nsub) const
+    bool is_subblock_null(unsigned nsub) const BMNOEXCEPT
     {
         BM_ASSERT(top_blocks_);
         if (nsub >= top_block_size_)
@@ -1740,14 +1751,14 @@ public:
         return top_blocks_[nsub] == NULL;
     }
 
-    bm::word_t*** top_blocks_root()
+    bm::word_t*** top_blocks_root() BMNOEXCEPT
     {
         return top_blocks_;
     }
 
     /*! \brief Returns current GAP level vector
     */
-    const gap_word_t* glen() const
+    const gap_word_t* glen() const BMNOEXCEPT
     {
         return glevel_len_;
     }
@@ -1755,14 +1766,14 @@ public:
     /*! \brief Returns GAP level length for specified level
         \param level - level number
     */
-    unsigned glen(unsigned level) const
+    unsigned glen(unsigned level) const BMNOEXCEPT
     {
         return glevel_len_[level];
     }
     
     /*! \brief Returns size of the top block array in the tree 
     */
-    unsigned top_block_size() const
+    unsigned top_block_size() const BMNOEXCEPT
     {
         return top_block_size_;
     }
@@ -1784,7 +1795,7 @@ public:
     */
     unsigned reserve_top_blocks(unsigned top_blocks)
     {
-        if (top_blocks_ && top_blocks <= top_block_size_)
+        if ((top_blocks_ && top_blocks <= top_block_size_) || !top_blocks )
             return top_block_size_; // nothing to do
         
         bm::word_t*** new_blocks = 
@@ -1793,8 +1804,16 @@ public:
         unsigned i = 0;
         if (top_blocks_)
         {
+            if (i < top_block_size_)
+            {
+                ::memcpy(&new_blocks[0], &top_blocks_[0],
+                            top_block_size_ * sizeof(top_blocks_[0]));
+                i = top_block_size_;
+            }
+            /*
             for (; i < top_block_size_; ++i)
                 new_blocks[i] = top_blocks_[i];
+            */
             alloc_.free_ptr(top_blocks_, top_block_size_);
         }
         if (i < top_blocks)
@@ -1808,21 +1827,20 @@ public:
     
     /** \brief Returns reference on the allocator
     */
-    allocator_type& get_allocator() { return alloc_; }
+    allocator_type& get_allocator() BMNOEXCEPT { return alloc_; }
 
     /** \brief Returns allocator
     */
-    allocator_type get_allocator() const { return alloc_; }
+    allocator_type get_allocator() const BMNOEXCEPT { return alloc_; }
     
     
     /// if tree of blocks already up
-    bool is_init() const { return top_blocks_ != 0; }
+    bool is_init() const BMNOEXCEPT { return top_blocks_ != 0; }
     
     /// allocate first level of descr. of blocks 
     void init_tree()
     {
         BM_ASSERT(top_blocks_ == 0);
-        
         if (top_block_size_)
         {
             top_blocks_ = (bm::word_t***) alloc_.alloc_ptr(top_block_size_);
@@ -1844,7 +1862,7 @@ public:
                 alloc_.free_bit_block(blk); \
         }
     
-    void deallocate_top_subblock(unsigned nblk_blk)
+    void deallocate_top_subblock(unsigned nblk_blk) BMNOEXCEPT
     {
         if (!top_blocks_[nblk_blk])
             return;
@@ -1886,7 +1904,7 @@ public:
     /** destroy tree, free memory in all blocks and control structures
         Note: pointers are NOT assigned to zero(!)
     */
-    void destroy_tree() BMNOEXEPT
+    void destroy_tree() BMNOEXCEPT
     {
         if (!top_blocks_)
             return;
@@ -1916,7 +1934,7 @@ public:
     }
     #undef BM_FREE_OP
 
-    void deinit_tree() BMNOEXEPT
+    void deinit_tree() BMNOEXCEPT
     {
         destroy_tree();
         top_blocks_ = 0; top_block_size_ = 0;
@@ -1925,7 +1943,7 @@ public:
     // ----------------------------------------------------------------
     
     /// calculate top blocks which are not NULL and not FULL
-    unsigned find_real_top_blocks() const
+    unsigned find_real_top_blocks() const BMNOEXCEPT
     {
         unsigned cnt = 0;
         unsigned top_blocks = top_block_size();
@@ -1943,7 +1961,7 @@ public:
     // ----------------------------------------------------------------
 
     /// calculate max top blocks size whithout NULL-tail
-    unsigned find_max_top_blocks() const
+    unsigned find_max_top_blocks() const BMNOEXCEPT
     {
         unsigned top_blocks = top_block_size();
         if (!top_blocks)
@@ -1960,11 +1978,11 @@ public:
 
     // ----------------------------------------------------------------
 
-    void validate_top_zero(unsigned i)
+    void validate_top_zero(unsigned i) BMNOEXCEPT
     {
         BM_ASSERT(i < top_block_size());
         bm::word_t** blk_blk = top_blocks_[i];
-        // TODO: SIMD
+        // TODO: SIMD or unroll
         for (unsigned j = 0; j < bm::set_sub_array_size; ++j)
         {
             if (blk_blk[j])
@@ -1976,7 +1994,7 @@ public:
 
     // ----------------------------------------------------------------
 
-    void validate_top_full(unsigned i)
+    void validate_top_full(unsigned i) BMNOEXCEPT
     {
         BM_ASSERT(i < top_block_size());
         bm::word_t** blk_blk = top_blocks_[i];
@@ -1994,7 +2012,7 @@ public:
         Calculate approximate memory needed to serialize big runs
         of 0000s and 111s (as blocks)
     */
-    size_t calc_serialization_null_full() const
+    size_t calc_serialization_null_full() const BMNOEXCEPT
     {
         size_t s_size = sizeof(unsigned);
         if (!top_blocks_)
@@ -2020,6 +2038,9 @@ public:
                 }
                 nb_empty += (i - nb_prev) * bm::set_sub_array_size;
                 blk_blk = top_blocks_[i];
+                BM_ASSERT(blk_blk);
+                if (!blk_blk)
+                    break;
             }
             if ((bm::word_t*)blk_blk == FULL_BLOCK_FAKE_ADDR)
             {
@@ -2057,6 +2078,7 @@ public:
         return s_size;
     }
 
+
     // ----------------------------------------------------------------
     
     void optimize_tree(bm::word_t*  temp_block, int opt_mode,
@@ -2228,14 +2250,20 @@ private:
               block_idx_type block_from = 0,
               block_idx_type block_to = bm::set_total_blocks)
     {
-        unsigned arg_top_blocks = blockman.top_block_size();
-        this->reserve_top_blocks(arg_top_blocks);
-        
-        bm::word_t*** blk_root = top_blocks_root();
         bm::word_t*** blk_root_arg = blockman.top_blocks_root();
-        
         if (!blk_root_arg)
             return;
+
+        unsigned arg_top_blocks = blockman.top_block_size();
+        {
+            block_idx_type need_top_blocks = 1 + (block_to / 256);
+            if (need_top_blocks < arg_top_blocks)
+                arg_top_blocks = unsigned(need_top_blocks);
+        }
+
+        this->reserve_top_blocks(arg_top_blocks);
+        bm::word_t*** blk_root = top_blocks_root();
+
         
         unsigned i_from, j_from, i_to, j_to;
         get_block_coord(block_from, i_from, j_from);
@@ -2329,7 +2357,7 @@ template<class BlocksManager>
 class bit_block_guard
 {
 public:
-    bit_block_guard(BlocksManager& bman, bm::word_t* blk=0) 
+    bit_block_guard(BlocksManager& bman, bm::word_t* blk=0) BMNOEXCEPT
         : bman_(bman), 
           block_(blk)
     {}
@@ -2338,18 +2366,20 @@ public:
         if (IS_VALID_ADDR(block_))
             bman_.get_allocator().free_bit_block(block_, 3);
     }
-    void attach(bm::word_t* blk)
+
+    void attach(bm::word_t* blk) BMNOEXCEPT
     {
         if (IS_VALID_ADDR(block_))
             bman_.get_allocator().free_bit_block(block_);
         block_ = blk;
     }
+
     bm::word_t* allocate()
     {
         attach(bman_.get_allocator().alloc_bit_block(3));
         return block_;
     }
-    bm::word_t* get() { return block_; }
+    bm::word_t* get() BMNOEXCEPT { return block_; }
 
 private:
     bit_block_guard(const bit_block_guard&);
