@@ -18,6 +18,9 @@ limitations under the License.
 For more information please visit:  http://bitmagic.io
 */
 
+#include <stddef.h>
+#include <type_traits>
+
 #include "bmdef.h"
 
 namespace bm
@@ -30,35 +33,35 @@ namespace bm
 class byte_buffer_ptr
 {
 public:
-    byte_buffer_ptr()
+    byte_buffer_ptr() BMNOEXCEPT
         : byte_buf_(0), size_(0)
     {}
     
     /// construct byte buffer pointer
     ///
-    byte_buffer_ptr(unsigned char* in_buf, size_t in_size)
+    byte_buffer_ptr(unsigned char* in_buf, size_t in_size) BMNOEXCEPT
         : byte_buf_(in_buf), size_(in_size)
     {}
     
     /// Set buffer pointer
-    void set_buf(unsigned char* in_buf, size_t in_size)
+    void set_buf(unsigned char* in_buf, size_t in_size) BMNOEXCEPT
     {
         byte_buf_ = in_buf; size_= in_size;
     }
 
     /// Get buffer size
-    size_t size() const { return size_; }
+    size_t size() const BMNOEXCEPT { return size_; }
     
     /// Get read access to buffer memory
-    const unsigned char* buf() const { return byte_buf_; }
+    const unsigned char* buf() const BMNOEXCEPT { return byte_buf_; }
 
     /// Get write access to buffer memory
-    unsigned char* data() { return byte_buf_; }
+    unsigned char* data() BMNOEXCEPT { return byte_buf_; }
 
-    bool operator==(const byte_buffer_ptr& lhs) const { return equal(lhs); }
+    bool operator==(const byte_buffer_ptr& lhs) const BMNOEXCEPT { return equal(lhs); }
     
     /// return true if content and size is the same
-    bool equal(const byte_buffer_ptr& lhs) const
+    bool equal(const byte_buffer_ptr& lhs) const BMNOEXCEPT
     {
         if (this == &lhs)
             return true;
@@ -86,7 +89,7 @@ public:
     typedef size_t                                           size_type;
     
 public:
-    byte_buffer() : capacity_(0), alloc_factor_(0)
+    byte_buffer() BMNOEXCEPT : capacity_(0), alloc_factor_(0)
     {}
     
     byte_buffer(size_t in_capacity)
@@ -95,7 +98,7 @@ public:
         allocate(in_capacity);
     }
     
-    byte_buffer(const byte_buffer& lhs)
+    byte_buffer(const byte_buffer& lhs) BMNOEXCEPT
     {
         byte_buf_ = 0;
         size_ = capacity_ = alloc_factor_ = 0;
@@ -107,7 +110,7 @@ public:
     
 #ifndef BM_NO_CXX11
     /// Move constructor
-    byte_buffer(byte_buffer&& in_buf) BMNOEXEPT
+    byte_buffer(byte_buffer&& in_buf) BMNOEXCEPT
     {
         byte_buf_ = in_buf.byte_buf_;
         in_buf.byte_buf_ = 0;
@@ -118,14 +121,14 @@ public:
     }
     
     /// Move assignment operator
-    byte_buffer& operator=(byte_buffer&& lhs) BMNOEXEPT
+    byte_buffer& operator=(byte_buffer&& lhs) BMNOEXCEPT
     {
         move_from(lhs);
         return *this;
     }
 #endif
 
-    byte_buffer& operator=(const byte_buffer& lhs)
+    byte_buffer& operator=(const byte_buffer& lhs) BMNOEXCEPT
     {
         if (this == &lhs)
             return *this;
@@ -140,7 +143,7 @@ public:
     }
     
     /// swap content with another buffer
-    void swap(byte_buffer& other) BMNOEXEPT
+    void swap(byte_buffer& other) BMNOEXCEPT
     {
         if (this == &other)
             return;
@@ -154,7 +157,7 @@ public:
     }
     
     /// take/move content from another buffer
-    void move_from(byte_buffer& other) BMNOEXEPT
+    void move_from(byte_buffer& other) BMNOEXCEPT
     {
         if (this == &other)
             return;
@@ -187,7 +190,7 @@ public:
     
     
     /// Get buffer capacity
-    size_t capacity() const { return capacity_; }
+    size_t capacity() const BMNOEXCEPT { return capacity_; }
 
     /// adjust current size (buffer content preserved)
     void resize(size_t new_size, bool copy_content = true)
@@ -210,6 +213,11 @@ public:
     {
         if (new_capacity <= capacity_)
             return;
+        if (!capacity_)
+        {
+            allocate(new_capacity);
+            return;
+        }
         
         byte_buffer tmp_buffer(new_capacity);
         tmp_buffer = *this;
@@ -244,7 +252,7 @@ public:
     }
     
     /// return memory consumtion
-    size_t mem_usage() const
+    size_t mem_usage() const BMNOEXCEPT
     {
         return sizeof(capacity_) + sizeof(alloc_factor_) +
                capacity();
@@ -255,7 +263,7 @@ private:
     void set_buf(unsigned char* buf, size_t size);
 
     /// compute number of words for the desired capacity
-    static size_t compute_words(size_t capacity)
+    static size_t compute_words(size_t capacity) BMNOEXCEPT
     {
         size_t words = (capacity / sizeof(bm::word_t))+1;
         return words;
@@ -295,7 +303,7 @@ private:
     Simple heap allocated vector based on bvector allocator
     @internal
 */
-template<typename Val, typename BVAlloc>
+template<typename Val, typename BVAlloc, bool trivial_type>
 class heap_vector
 {
 public:
@@ -304,25 +312,24 @@ public:
     typedef Val                                              value_type;
     typedef typename buffer_type::size_type                  size_type;
 
-    heap_vector()
-        : buffer_()    
+    heap_vector() BMNOEXCEPT : buffer_()
     {}
 
-    heap_vector(const heap_vector<Val, BVAlloc>& hv)
+    heap_vector(const heap_vector<Val, BVAlloc, trivial_type>& hv)
         : buffer_()
     {
         size_type v_size = value_size();
         size_type new_size = hv.size();
         buffer_.resize(new_size * v_size);
-        unsigned char* data = buffer_.data();
+        unsigned char* this_data = buffer_.data();
         for (size_type i = 0; i < new_size; ++i)
         {
-            unsigned char *p = data + (i * v_size);
+            unsigned char *p = this_data + (i * v_size);
             new(p) value_type(hv[i]);
         }
     }
 
-    heap_vector& operator=(const heap_vector<Val, BVAlloc>& hv)
+    heap_vector& operator=(const heap_vector<Val, BVAlloc, trivial_type>& hv)
     {
         if (this == &hv)
             return *this;
@@ -331,10 +338,10 @@ public:
         size_type v_size = value_size();
         size_type new_size = hv.size();
         buffer_.resize(new_size * v_size);
-        unsigned char* data = buffer_.data();
+        unsigned char* this_data = buffer_.data();
         for (size_type i = 0; i < new_size; ++i)
         {
-            unsigned char *p = data + (i * v_size);
+            unsigned char *p = this_data + (i * v_size);
             new(p) value_type(hv[i]);
         }
         return *this;
@@ -342,24 +349,27 @@ public:
    
     ~heap_vector()
     {
-        size_type sz = size();
-        size_type v_size = value_size();
-        unsigned char* data = buffer_.data();
-        for (size_type i = 0; i < sz; ++i) 
+        if (!trivial_type)
         {
-            unsigned char *p = data + (i * v_size);
-            reinterpret_cast<value_type*>(p)->~Val();
+            size_type sz = size();
+            size_type v_size = value_size();
+            unsigned char* this_data = buffer_.data();
+            for (size_type i = 0; i < sz; ++i)
+            {
+                unsigned char *p = this_data + (i * v_size);
+                reinterpret_cast<value_type*>(p)->~Val();
+            }
         }
     }
     
-    value_type* data() { return (value_type*) buffer_.data(); }
+    value_type* data() BMNOEXCEPT { return (value_type*) buffer_.data(); }
 
-    void swap(heap_vector<Val, BVAlloc>& other) BMNOEXEPT
+    void swap(heap_vector<Val, BVAlloc, trivial_type>& other) BMNOEXCEPT
     {
         buffer_.swap(other.buffer_);
     }
 
-    const value_type& operator[](std::size_t pos) const
+    const value_type& operator[](size_type pos) const BMNOEXCEPT
     {
         BM_ASSERT(pos < size());
         size_type v_size = value_size();
@@ -367,7 +377,7 @@ public:
         return *reinterpret_cast<const value_type*>(p);
     }
 
-    value_type& operator[](std::size_t pos)
+    value_type& operator[](size_type pos) BMNOEXCEPT
     {
         BM_ASSERT(pos < size());
         size_type v_size = value_size();
@@ -375,7 +385,7 @@ public:
         return *reinterpret_cast<value_type*>(p);
     }
 
-    value_type& at(std::size_t pos)
+    value_type& at(size_type pos)
     {
         size_type sz = size();
         if (pos >= sz)
@@ -386,22 +396,22 @@ public:
         return *reinterpret_cast<value_type*>(p);
     }
     
-    const value_type* begin() const
+    const value_type* begin() const BMNOEXCEPT
     {
         return (const value_type*) buffer_.buf();
     }
 
-    size_type size() const
+    size_type size() const BMNOEXCEPT
     {
         return buffer_.size() / value_size();
     }
 
-    size_type capacity() const
+    size_type capacity() const BMNOEXCEPT
     {
         return buffer_.capacity() / value_size();
     }
 
-    bool empty() const
+    bool empty() const BMNOEXCEPT
     {
         return (buffer_.size() == 0);
     }
@@ -415,6 +425,12 @@ public:
         buffer_.reserve(new_size * v_size);
     }
 
+    /**
+        @brief vector resize
+        @param new_size - new number of elements
+        @param init_destroy_values - need to init or destroy values 
+           false - skip construction/destruction
+    */
     void resize(size_type new_size)
     {
         size_type sz = size();
@@ -423,22 +439,28 @@ public:
             return;
         if (new_size < sz) // shrink
         {
-            unsigned char* data = buffer_.data();
-            for (size_type i = new_size; i < sz; ++i)
+            if (!trivial_type)
             {
-                unsigned char *p = data + (i * v_size);
-                reinterpret_cast<value_type*>(p)->~Val();
+                unsigned char* this_data = buffer_.data();
+                for (size_type i = new_size; i < sz; ++i)
+                {
+                    unsigned char *p = this_data + (i * v_size);
+                    reinterpret_cast<value_type*>(p)->~Val();
+                }
             }
             buffer_.resize(new_size * v_size);
         }
         else
         {
             buffer_.resize(new_size * v_size);
-            unsigned char* data = buffer_.data();
-            for (size_type i = sz; i < new_size; ++i)
+            if (!trivial_type)
             {
-                unsigned char *p = data + (i * v_size);
-                new(p) value_type();
+                unsigned char* this_data = buffer_.data();
+                for (size_type i = sz; i < new_size; ++i)
+                {
+                    unsigned char *p = this_data + (i * v_size);
+                    new(p) value_type();
+                }
             }
         }
     }
@@ -473,7 +495,7 @@ protected:
         buffer_.resize(new_size * v_size);
     }
 
-    static size_type value_size()
+    static size_type value_size() BMNOEXCEPT
     {
         size_type size_of = sizeof(value_type);
         return size_of;
@@ -484,6 +506,7 @@ protected:
     #ifndef BM_NO_STL
         throw std::range_error(err_msg);
     #else
+        (void) err_msg;
         BM_ASSERT_THROW(false, BM_ERR_RANGE);
     #endif
     }
@@ -517,13 +540,13 @@ public:
         row_size_in_bytes = sizeof(value_type) * COLS
     };
 
-    static size_t rows() { return ROWS; }
-    static size_t cols() { return COLS; }
+    static size_t rows() BMNOEXCEPT { return ROWS; }
+    static size_t cols() BMNOEXCEPT { return COLS; }
 
     /**
         By default object is constructed NOT allocated.
     */
-    heap_matrix()
+    heap_matrix() BMNOEXCEPT
         : buffer_()
     {}
 
@@ -545,12 +568,12 @@ public:
         buffer_.resize(size_in_bytes);
     }
     
-    bool is_init() const
+    bool is_init() const BMNOEXCEPT
     {
         return buffer_.size();
     }
 
-    value_type get(size_t row_idx, size_t col_idx) const
+    value_type get(size_type row_idx, size_type col_idx) const BMNOEXCEPT
     {
         BM_ASSERT(row_idx < ROWS);
         BM_ASSERT(col_idx < COLS);
@@ -559,7 +582,7 @@ public:
         return ((const value_type*)buf)[col_idx];
     }
 
-    const value_type* row(size_t row_idx) const
+    const value_type* row(size_type row_idx) const BMNOEXCEPT
     {
         BM_ASSERT(row_idx < ROWS);
         BM_ASSERT(buffer_.size());
@@ -567,7 +590,7 @@ public:
         return (const value_type*) buf;
     }
 
-    value_type* row(size_t row_idx)
+    value_type* row(size_type row_idx) BMNOEXCEPT
     {
         BM_ASSERT(row_idx < ROWS);
         BM_ASSERT(buffer_.size());
@@ -577,21 +600,21 @@ public:
     }
 
     /** memset all buffer to all zeroes */
-    void set_zero()
+    void set_zero() BMNOEXCEPT
     {
         ::memset(buffer_.data(), 0, size_in_bytes);
     }
     
     /*!  swap content
     */
-    void swap(heap_matrix& other) BMNOEXEPT
+    void swap(heap_matrix& other) BMNOEXCEPT
     {
         buffer_.swap(other.buffer_);
     }
     
     /*!  move content from another matrix
     */
-    void move_from(heap_matrix& other) BMNOEXEPT
+    void move_from(heap_matrix& other) BMNOEXCEPT
     {
         buffer_.move_from(other.buffer_);
     }
@@ -604,7 +627,7 @@ public:
     /*! remapping: vect[idx] = matrix[idx, vect[idx] ]
     */
     template<typename VECT_TYPE>
-    void remap(VECT_TYPE* vect, size_type size) const
+    void remap(VECT_TYPE* vect, size_type size) const BMNOEXCEPT
     {
         BM_ASSERT(size <= ROWS);
         const unsigned char* buf = buffer_.buf();
@@ -621,7 +644,7 @@ public:
     /*! zero-terminated remap: vect[idx] = matrix[idx, vect[idx] ]
     */
     template<typename VECT_TYPE>
-    void remapz(VECT_TYPE* vect) const
+    void remapz(VECT_TYPE* vect) const BMNOEXCEPT
     {
         const unsigned char* buf = buffer_.buf();
         for (size_type i = 0; i < ROWS; ++i)
@@ -661,8 +684,8 @@ public:
     /**
         By default object is constructed but NOT allocated.
     */
-    dynamic_heap_matrix(size_type rows=0, size_type cols=0)
-        : rows_(rows), cols_(cols),
+    dynamic_heap_matrix(size_type rows_in=0, size_type cols_in=0)
+        : rows_(rows_in), cols_(cols_in),
         buffer_()
     {}
 
@@ -678,18 +701,18 @@ public:
     size_type rows() const { return rows_; }
     size_type cols() const { return cols_; }
 
-    void resize(size_type rows, size_type cols)
+    void resize(size_type rows_in, size_type cols_in)
     {
-        rows_ = rows; cols_ = cols;
+        rows_ = rows_in; cols_ = cols_in;
         buffer_.resize(size_in_bytes());
     }
     
-    bool is_init() const
+    bool is_init() const BMNOEXCEPT
     {
         return buffer_.size();
     }
 
-    const value_type* row(size_type row_idx) const
+    const value_type* row(size_type row_idx) const BMNOEXCEPT
     {
         BM_ASSERT(row_idx < rows_);
         BM_ASSERT(buffer_.size());
@@ -697,7 +720,7 @@ public:
         return (const value_type*) buf;
     }
 
-    value_type* row(size_type row_idx)
+    value_type* row(size_type row_idx) BMNOEXCEPT
     {
         BM_ASSERT(row_idx < rows_);
         BM_ASSERT(buffer_.size());
@@ -707,14 +730,14 @@ public:
     }
 
     /** memset all buffer to all zeroes */
-    void set_zero()
+    void set_zero() BMNOEXCEPT
     {
         ::memset(buffer_.data(), 0, size_in_bytes());
     }
     
     /*!  swap content
     */
-    void swap(dynamic_heap_matrix& other) BMNOEXEPT
+    void swap(dynamic_heap_matrix& other) BMNOEXCEPT
     {
         bm::xor_swap(rows_, other.rows_);
         bm::xor_swap(cols_, other.cols_);
@@ -723,7 +746,7 @@ public:
     
     /*!  move content from another matrix
     */
-    void move_from(dynamic_heap_matrix& other) BMNOEXEPT
+    void move_from(dynamic_heap_matrix& other) BMNOEXCEPT
     {
         rows_ = other.rows_;
         cols_ = other.cols_;
@@ -731,16 +754,17 @@ public:
     }
 
     /** Get low-level buffer access */
-    buffer_type& get_buffer() { return buffer_; }
+    buffer_type& get_buffer() BMNOEXCEPT { return buffer_; }
     /** Get low-level buffer access */
-    const buffer_type& get_buffer() const { return buffer_; }
+    const buffer_type& get_buffer() const BMNOEXCEPT { return buffer_; }
 
 protected:
-    size_type size_in_bytes() const
+
+    size_type size_in_bytes() const BMNOEXCEPT
     {
         return sizeof(value_type) * cols_ * rows_;
     }
-    size_type row_size_in_bytes() const
+    size_type row_size_in_bytes() const BMNOEXCEPT
     {
         return sizeof(value_type) * cols_;
     }
