@@ -418,7 +418,7 @@ void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
     for (nb = 0; nb < blocks; ++nb)
     {
         unsigned i0, j0;
-        bman.get_block_coord(nb, i0, j0);
+        bm::get_block_coord(nb, i0, j0);
         const bm::word_t* blk = bman.get_block(i0, j0);
 
         if (!blk)
@@ -435,7 +435,7 @@ void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
            typename BV::block_idx_type start = nb;
            for(auto i = nb+1; i < bm::set_total_blocks; ++i, ++nb)
            {
-               bman.get_block_coord(nb, i0, j0);
+               bm::get_block_coord(nb, i0, j0);
                blk = bman.get_block(i0, j0);
                if (IS_FULL_BLOCK(blk))
                {
@@ -478,7 +478,7 @@ void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
                total_gap_eff += unsigned(mem_eff);
                
                unsigned i,j;
-               bman.get_block_coord(nb, i, j);
+               bm::get_block_coord(nb, i, j);
                std::cout << " [GAP " << nb << "(" << i << "," << j << ")"
                          << "=" << bc << ":" << level << "-L" << len << "(" << mem_eff << ")]";
                 ++printed;
@@ -510,15 +510,15 @@ void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
 }
 
 template<class BV>
-unsigned compute_serialization_size(const BV& bv)
+size_t compute_serialization_size(const BV& bv)
 {
     BM_DECLARE_TEMP_BLOCK(tb)
     unsigned char*  buf = 0;
-    unsigned blob_size = 0;
+    typename BV::size_type blob_size = 0;
     try
     {
         bm::serializer<BV> bvs(typename BV::allocator_type(), tb);
-        bvs.set_compression_level(4);
+        //bvs.set_compression_level(4);
         
         typename BV::statistics st;
         bv.calc_stat(&st);
@@ -536,7 +536,94 @@ unsigned compute_serialization_size(const BV& bv)
     return blob_size;
 }
 
+#if 0
+template<class SV>
+void print_svector_xor_stat(const SV& sv)
+{
+    BM_DECLARE_TEMP_BLOCK(tb)
+    typename SV::size_type sz = sv.size();
+    if (!sz)
+        return;
+    typename SV::size_type nb_max = (sz >>  bm::set_block_shift);
 
+    for (typename SV::size_type nb = 0; nb < nb_max; ++nb)
+    {
+        std::cout << "nb = " << nb << std::endl;
+
+        unsigned i0 = unsigned(nb >> bm::set_array_shift);
+        unsigned j0 = unsigned(nb &  bm::set_array_mask);
+
+        auto plains = sv.plains();
+        for (unsigned i = 0; i < plains; ++i)
+        {
+            const typename SV::bvector_type* bv = sv.get_plain(i);
+            if (!bv)
+                continue;
+            const typename SV::bvector_type::blocks_manager_type& bman = bv->get_blocks_manager();
+            const bm::word_t* block = bman.get_block_ptr(i0, j0);
+            if (!IS_VALID_ADDR(block) || BM_IS_GAP(block))
+                continue;
+
+            // compute block complexity
+            bm::block_waves_xor_descr  x_descr;
+            bm::compute_complexity_descr(block, x_descr);
+            unsigned gc, bc;
+            bm::bit_block_change_bc32(block, &gc, &bc);
+            unsigned best_metric, block_metric;
+            block_metric = best_metric = gc < bc ? gc : bc;
+
+            bool kb_found = false;
+            bm::id64_t d64 = 0;
+            for (unsigned k = i + 1; k < plains; ++k)
+            {
+                const typename SV::bvector_type* bv_x = sv.get_plain(i);
+                if (!bv_x)
+                    continue;
+                const typename SV::bvector_type::blocks_manager_type& bman_x = bv_x->get_blocks_manager();
+                const bm::word_t* block_x = bman_x.get_block_ptr(i0, j0);
+                if (!IS_VALID_ADDR(block_x) || BM_IS_GAP(block_x))
+                    continue;
+
+                // evaluate potential key block as XOR filter
+                bm::id64_t kb_d64 =
+                    bm::compute_xor_complexity_descr(block, block_x, x_descr);
+                if (kb_d64) // candidate XOR filter found
+                {
+                    bm::bit_block_xor_product(tb, block, block_x, kb_d64);
+                    unsigned kb_bc, kb_gc;
+                    bm::bit_block_change_bc32(tb, &kb_gc, &kb_bc);
+                    if (kb_gc < best_metric && kb_gc < bm::bie_cut_off)
+                    {
+                        d64 = kb_d64;
+                        best_metric = kb_gc;
+                        kb_found = true;
+                        //*kb_j = j0;
+                    }
+                    if (kb_bc < best_metric && kb_bc < bm::bie_cut_off)
+                    {
+                        d64 = kb_d64;
+                        best_metric = kb_bc;
+                        kb_found = true;
+                        //*kb_j = j0;
+                    }
+
+                }
+
+            } // for k
+
+            if (kb_found)
+            {
+                std::cout << "XOR match " << "metric gain = " << std::endl;
+            }
+
+
+            std::cout << std::endl;
+
+        } // for i
+
+    } // for nb
+}
+#endif
 
 template<class SV>
 void print_svector_stat(const SV& svect, bool print_sim = false)
@@ -590,12 +677,12 @@ void print_svector_stat(const SV& svect, bool print_sim = false)
                 const typename SV::bvector_type* bv1 = sim_vec[k].get_first();
                 const typename SV::bvector_type* bv2 = sim_vec[k].get_second();
 
-                unsigned bv_size2 = compute_serialization_size(*bv2);
+                auto bv_size2 = compute_serialization_size(*bv2);
                 
                 typename SV::bvector_type bvx(*bv2);
                 bvx ^= *bv1;
                 
-                unsigned bv_size_x = compute_serialization_size(bvx);
+                auto bv_size_x = compute_serialization_size(bvx);
                 if (bv_size_x < bv_size2) // true savings
                 {
                     size_t diff = bv_size2 - bv_size_x;
@@ -817,10 +904,15 @@ int file_save_svector(const SV& sv, const std::string& fname, size_t* sv_blob_si
     BM_ASSERT(!fname.empty());
     
     bm::sparse_vector_serial_layout<SV> sv_lay;
-    
+
+    bm::sparse_vector_serializer<SV> sv_serializer;
+    sv_serializer.set_xor_ref(true);
+
+    sv_serializer.serialize(sv, sv_lay);
+/*
     BM_DECLARE_TEMP_BLOCK(tb)
     bm::sparse_vector_serialize(sv, sv_lay, tb);
-
+*/
     std::ofstream fout(fname.c_str(), std::ios::binary);
     if (!fout.good())
     {
@@ -869,7 +961,7 @@ int file_load_svector(SV& sv, const std::string& fname)
 }
 
 
-// comapre-check if sparse vector is excatly coresponds to vector 
+// compare-check if sparse vector is excatly coresponds to vector
 //
 // returns 0 - if equal
 //         1 - no size match
@@ -892,6 +984,20 @@ int svector_check(const SV& sv, const V& vect)
 }
 
 
+template<class SV, class BV>
+void convert_bv2sv(SV& sv, const BV& bv)
+{
+    typename SV::back_insert_iterator bit = sv.get_back_inserter();
+    typename BV::enumerator en = bv.first();
+    for (; en.valid(); ++en)
+    {
+        auto v = en.value();
+        bit = v;
+    }
+    bit.flush();
+}
+
+
 } // namespace
 
 #include "bmundef.h"
