@@ -71,19 +71,32 @@ For more information please visit:  http://bitmagic.io
 
 // cxx11 features
 //
-#if defined(BM_NO_CXX11)  ||  (defined(_MSC_VER)  &&  _MSC_VER < 1900)
-# define BMNOEXEPT
+#if defined(BM_NO_CXX11) || (defined(_MSC_VER)  &&  _MSC_VER < 1900)
+# define BMNOEXCEPT
+# define BMNOEXCEPT2
 #else
-# ifndef BMNOEXEPT
-#  define BMNOEXEPT noexcept
+# ifndef BMNOEXCEPT
+#  define BMNOEXCEPT noexcept
+#if defined(__EMSCRIPTEN__)
+#else
+#  define BMNOEXCEPT2
+#endif
 # endif
 #endif
 
-
-// disable 'register' keyword, which is obsolete in C++11
+// WebAssembly compilation settings
 //
-#ifndef BMREGISTER
-# define BMREGISTER
+// detects use of EMSCRIPTEN engine and tweaks settings
+// WebAssemply compiles into 32-bit ptr yet 64-bit wordsize use GCC extensions
+//
+// BMNOEXCEPT2 is to declare "noexcept" for WebAsm only where needed
+// and silence GCC warnings where not
+#if defined(__EMSCRIPTEN__)
+# define BM64OPT
+# define BM_USE_GCC_BUILD
+# define BMNOEXCEPT2 noexcept
+#else
+#  define BMNOEXCEPT2
 #endif
 
 
@@ -389,6 +402,23 @@ For more information please visit:  http://bitmagic.io
 #endif
 
 
+#ifndef __has_cpp_attribute
+#  define __has_cpp_attribute(x) 0
+#endif
+#ifndef __has_attribute
+#  define __has_attribute(x) 0
+#endif
+#if __has_cpp_attribute(fallthrough)
+#  define BM_FALLTHROUGH [[fallthrough]]
+#elif __has_cpp_attribute(gcc::fallthrough)
+#  define BM_FALLTHROUGH [[gcc::fallthrough]]
+#elif __has_cpp_attribute(clang::fallthrough)
+#  define BM_FALLTHROUGH [[clang::fallthrough]]
+#elif __has_attribute(fallthrough)
+#  define BM_FALLTHROUGH __attribute__ ((fallthrough))
+#else
+#  define BM_FALLTHROUGH
+#endif
 
 #endif
 
