@@ -576,6 +576,17 @@ bool sse4_is_all_one(const __m128i* BMRESTRICT block)
     return true;
 }
 
+/*!
+    @brief check if SSE wave is all oxFFFF...FFF
+    @ingroup SSE4
+*/
+BMFORCEINLINE
+bool sse42_test_all_one_wave(const void* ptr)
+{
+    return _mm_test_all_ones(_mm_loadu_si128((__m128i*)ptr));
+}
+
+
 /*!
     @brief check if wave of pointers is all NULL
     @ingroup SSE4
@@ -619,10 +630,11 @@ bool sse42_test_all_eq_wave2(const void* ptr0, const void* ptr1)
     @ingroup SSE4
 */
 inline
-unsigned sse42_bit_block_calc_change(const __m128i* BMRESTRICT block)
+unsigned sse42_bit_block_calc_change(const __m128i* BMRESTRICT block,
+                                     unsigned size)
 {
     const __m128i* block_end =
-        ( __m128i*)((bm::word_t*)(block) + bm::set_block_size);
+        ( __m128i*)((bm::word_t*)(block) + size); // bm::set_block_size
     __m128i m1COshft, m2COshft;
     
     unsigned w0 = *((bm::word_t*)(block));
@@ -691,6 +703,280 @@ unsigned sse42_bit_block_calc_change(const __m128i* BMRESTRICT block)
 }
 
 
+/*!
+    SSE4.2 calculate number of bit changes from 0 to 1 of a XOR product
+    @ingroup SSE4
+*/
+inline
+unsigned sse42_bit_block_calc_xor_change(const __m128i* BMRESTRICT block,
+                                     const __m128i* BMRESTRICT xor_block,
+                                     unsigned size)
+{
+    const __m128i* block_end =
+        ( __m128i*)((bm::word_t*)(block) + size);
+    __m128i m1COshft, m2COshft;
+
+    unsigned w0 = *((bm::word_t*)(block));
+    unsigned count = 1;
+
+    unsigned co2, co1 = 0;
+    for (;block < block_end; block += 2, xor_block += 2)
+    {
+        __m128i m1A = _mm_load_si128(block);
+        __m128i m2A = _mm_load_si128(block+1);
+        __m128i m1B = _mm_load_si128(xor_block);
+        __m128i m2B = _mm_load_si128(xor_block+1);
+
+        m1A = _mm_xor_si128(m1A, m1B);
+        m2A = _mm_xor_si128(m2A, m2B);
+
+        __m128i m1CO = _mm_srli_epi32(m1A, 31);
+        __m128i m2CO = _mm_srli_epi32(m2A, 31);
+
+        co2 = _mm_extract_epi32(m1CO, 3);
+
+        __m128i m1As = _mm_slli_epi32(m1A, 1); // (block[i] << 1u)
+        __m128i m2As = _mm_slli_epi32(m2A, 1);
+
+        m1COshft = _mm_slli_si128 (m1CO, 4); // byte shift left by 1 int32
+        m1COshft = _mm_insert_epi32 (m1COshft, co1, 0);
+
+        co1 = co2;
+
+        co2 = _mm_extract_epi32(m2CO, 3);
+
+        m2COshft = _mm_slli_si128 (m2CO, 4);
+        m2COshft = _mm_insert_epi32 (m2COshft, co1, 0);
+
+        m1As = _mm_or_si128(m1As, m1COshft); // block[i] |= co_flag
+        m2As = _mm_or_si128(m2As, m2COshft);
+
+        co1 = co2;
+
+        // we now have two shifted SSE4 regs with carry-over
+        m1A = _mm_xor_si128(m1A, m1As); // w ^= (w >> 1);
+        m2A = _mm_xor_si128(m2A, m2As);
+
+#ifdef BM64_SSE4
+        bm::id64_t m0 = _mm_extract_epi64(m1A, 0);
+        bm::id64_t m1 = _mm_extract_epi64(m1A, 1);
+        count += unsigned(_mm_popcnt_u64(m0) + _mm_popcnt_u64(m1));
+
+        m0 = _mm_extract_epi64(m2A, 0);
+        m1 = _mm_extract_epi64(m2A, 1);
+        count += unsigned(_mm_popcnt_u64(m0) + _mm_popcnt_u64(m1));
+#else
+        bm::id_t m0 = _mm_extract_epi32(m1A, 0);
+        bm::id_t m1 = _mm_extract_epi32(m1A, 1);
+        bm::id_t m2 = _mm_extract_epi32(m1A, 2);
+        bm::id_t m3 = _mm_extract_epi32(m1A, 3);
+        count += unsigned(_mm_popcnt_u32(m0) + _mm_popcnt_u32(m1) +
+                          _mm_popcnt_u32(m2) + _mm_popcnt_u32(m3));
+
+        m0 = _mm_extract_epi32(m2A, 0);
+        m1 = _mm_extract_epi32(m2A, 1);
+        m2 = _mm_extract_epi32(m2A, 2);
+        m3 = _mm_extract_epi32(m2A, 3);
+        count += unsigned(_mm_popcnt_u32(m0) + _mm_popcnt_u32(m1) +
+                          _mm_popcnt_u32(m2) + _mm_popcnt_u32(m3));
+#endif
+
+    }
+    count -= (w0 & 1u); // correct initial carry-in error
+    return count;
+}
+
+
+
+#ifdef BM64_SSE4
+
+/*!
+    SSE4.2 calculate number of bit changes from 0 to 1
+    @ingroup SSE4
+*/
+inline
+void sse42_bit_block_calc_change_bc(const __m128i* BMRESTRICT block,
+                                    unsigned* gc, unsigned* bc)
+{
+    const __m128i* block_end =
+        ( __m128i*)((bm::word_t*)(block) + bm::set_block_size);
+    __m128i m1COshft, m2COshft;
+    
+    unsigned w0 = *((bm::word_t*)(block));
+    unsigned bit_count = 0;
+    unsigned gap_count = 1;
+
+    unsigned co2, co1 = 0;
+    for (;block < block_end; block += 2)
+    {
+        __m128i m1A = _mm_load_si128(block);
+        __m128i m2A = _mm_load_si128(block+1);
+        {
+            bm::id64_t m0 = _mm_extract_epi64(m1A, 0);
+            bm::id64_t m1 = _mm_extract_epi64(m1A, 1);
+            bit_count += unsigned(_mm_popcnt_u64(m0) + _mm_popcnt_u64(m1));
+            m0 = _mm_extract_epi64(m2A, 0);
+            m1 = _mm_extract_epi64(m2A, 1);
+            bit_count += unsigned(_mm_popcnt_u64(m0) + _mm_popcnt_u64(m1));
+        }
+
+        __m128i m1CO = _mm_srli_epi32(m1A, 31);
+        __m128i m2CO = _mm_srli_epi32(m2A, 31);
+        
+        co2 = _mm_extract_epi32(m1CO, 3);
+        
+        __m128i m1As = _mm_slli_epi32(m1A, 1); // (block[i] << 1u)
+        __m128i m2As = _mm_slli_epi32(m2A, 1);
+        
+        m1COshft = _mm_slli_si128 (m1CO, 4); // byte shift left by 1 int32
+        m1COshft = _mm_insert_epi32 (m1COshft, co1, 0);
+        
+        co1 = co2;
+        
+        co2 = _mm_extract_epi32(m2CO, 3);
+        
+        m2COshft = _mm_slli_si128 (m2CO, 4);
+        m2COshft = _mm_insert_epi32 (m2COshft, co1, 0);
+        
+        m1As = _mm_or_si128(m1As, m1COshft); // block[i] |= co_flag
+        m2As = _mm_or_si128(m2As, m2COshft);
+        
+        co1 = co2;
+        
+        // we now have two shifted SSE4 regs with carry-over
+        m1A = _mm_xor_si128(m1A, m1As); // w ^= (w >> 1);
+        m2A = _mm_xor_si128(m2A, m2As);
+        {
+            bm::id64_t m0 = _mm_extract_epi64(m1A, 0);
+            bm::id64_t m1 = _mm_extract_epi64(m1A, 1);
+            gap_count += unsigned(_mm_popcnt_u64(m0) + _mm_popcnt_u64(m1));
+        }
+
+        bm::id64_t m0 = _mm_extract_epi64(m2A, 0);
+        bm::id64_t m1 = _mm_extract_epi64(m2A, 1);
+        gap_count += unsigned(_mm_popcnt_u64(m0) + _mm_popcnt_u64(m1));
+
+    }
+    gap_count -= (w0 & 1u); // correct initial carry-in error
+    *gc = gap_count;
+    *bc = bit_count;
+}
+
+#endif
+
+
+/*!
+   \brief Find first bit which is different between two bit-blocks
+  @ingroup AVX2
+*/
+inline
+bool sse42_bit_find_first_diff(const __m128i* BMRESTRICT block1,
+                               const __m128i* BMRESTRICT block2,
+                               unsigned* pos)
+{
+    unsigned BM_ALIGN32 simd_buf[4] BM_ALIGN32ATTR;
+
+    const __m128i* block1_end =
+        (const __m128i*)((bm::word_t*)(block1) + bm::set_block_size);
+    __m128i maskZ = _mm_setzero_si128();
+    __m128i mA, mB;
+    unsigned simd_lane = 0;
+    do
+    {
+        mA = _mm_xor_si128(_mm_load_si128(block1), _mm_load_si128(block2));
+        mB = _mm_xor_si128(_mm_load_si128(block1+1), _mm_load_si128(block2+1));
+        __m128i mOR = _mm_or_si128(mA, mB);
+        if (!_mm_test_all_zeros(mOR, mOR)) // test 2x128 lanes
+        {
+            if (!_mm_test_all_zeros(mA, mA))
+            {
+                unsigned mask = _mm_movemask_epi8(_mm_cmpeq_epi32(mA, maskZ));
+                mask = ~mask; // invert to find (w != 0)
+                BM_ASSERT(mask);
+                int bsf = bm::bsf_asm32(mask); // find first !=0 (could use lzcnt())
+                _mm_store_si128 ((__m128i*)simd_buf, mA);
+                unsigned widx = bsf >> 2; // (bsf / 4);
+                unsigned w = simd_buf[widx]; // _mm_extract_epi32 (mA, widx);
+                bsf = bm::bsf_asm32(w); // find first bit != 0
+                *pos = (simd_lane * 128) + (widx * 32) + bsf;
+                return true;
+            }
+            unsigned mask = _mm_movemask_epi8(_mm_cmpeq_epi32(mB, maskZ));
+            mask = ~mask; // invert to find (w != 0)
+            BM_ASSERT(mask);
+            int bsf = bm::bsf_asm32(mask); // find first !=0 (could use lzcnt())
+            _mm_store_si128 ((__m128i*)simd_buf, mB);
+            unsigned widx = bsf >> 2; // (bsf / 4);
+            unsigned w = simd_buf[widx]; // _mm_extract_epi32 (mB, widx);
+            bsf = bm::bsf_asm32(w); // find first bit != 0
+            *pos = ((++simd_lane) * 128) + (widx * 32) + bsf;
+            return true;
+        }
+
+        simd_lane+=2;
+        block1+=2; block2+=2;
+
+    } while (block1 < block1_end);
+    return false;
+}
+
+
+/*!
+   \brief Find first non-zero bit
+  @ingroup AVX2
+*/
+inline
+bool sse42_bit_find_first(const __m128i* BMRESTRICT block,
+                          unsigned* pos)
+{
+    unsigned BM_ALIGN32 simd_buf[4] BM_ALIGN32ATTR;
+
+    const __m128i* block_end =
+        (const __m128i*)((bm::word_t*)(block) + bm::set_block_size);
+    __m128i maskZ = _mm_setzero_si128();
+    __m128i mA, mB;
+    unsigned simd_lane = 0;
+    do
+    {
+        mA = _mm_load_si128(block); mB = _mm_load_si128(block+1);
+        __m128i mOR = _mm_or_si128(mA, mB);
+        if (!_mm_test_all_zeros(mOR, mOR)) // test 2x128 lanes
+        {
+            if (!_mm_test_all_zeros(mA, mA))
+            {
+                unsigned mask = _mm_movemask_epi8(_mm_cmpeq_epi32(mA, maskZ));
+                mask = ~mask; // invert to find (w != 0)
+                BM_ASSERT(mask);
+                int bsf = bm::bsf_asm32(mask); // find first !=0 (could use lzcnt())
+                _mm_store_si128 ((__m128i*)simd_buf, mA);
+                unsigned widx = bsf >> 2; // (bsf / 4);
+                unsigned w = simd_buf[widx];
+                bsf = bm::bsf_asm32(w); // find first bit != 0
+                *pos = (simd_lane * 128) + (widx * 32) + bsf;
+                return true;
+            }
+            unsigned mask = _mm_movemask_epi8(_mm_cmpeq_epi32(mB, maskZ));
+            mask = ~mask; // invert to find (w != 0)
+            BM_ASSERT(mask);
+            int bsf = bm::bsf_asm32(mask); // find first !=0 (could use lzcnt())
+            _mm_store_si128 ((__m128i*)simd_buf, mB);
+            unsigned widx = bsf >> 2; // (bsf / 4);
+            unsigned w = simd_buf[widx];
+            bsf = bm::bsf_asm32(w); // find first bit != 0
+            *pos = ((++simd_lane) * 128) + (widx * 32) + bsf;
+            return true;
+        }
+
+        simd_lane+=2;
+        block+=2;
+
+    } while (block < block_end);
+    return false;
+}
+
+
+
+
 #ifdef __GNUG__
 // necessary measure to silence false warning from GCC about negative pointer arithmetics
 #pragma GCC diagnostic push
@@ -698,12 +984,14 @@ unsigned sse42_bit_block_calc_change(const __m128i* BMRESTRICT block)
 #endif
 
 /*!
-     SSE4.2 check for one to two (variable len) 128 bit SSE lines for gap search results (8 elements)
+     SSE4.2 check for one to two (variable len) 128 bit SSE lines
+     for gap search results (8 elements)
      @ingroup SSE4
      \internal
 */
 inline
-unsigned sse4_gap_find(const bm::gap_word_t* BMRESTRICT pbuf, const bm::gap_word_t pos, const unsigned size)
+unsigned sse4_gap_find(const bm::gap_word_t* BMRESTRICT pbuf,
+                       const bm::gap_word_t pos, const unsigned size)
 {
     BM_ASSERT(size <= 16);
     BM_ASSERT(size);
@@ -756,6 +1044,74 @@ unsigned sse4_gap_find(const bm::gap_word_t* BMRESTRICT pbuf, const bm::gap_word
     return size - bc;
 }
 
+/**
+    Hybrid binary search, starts as binary, then switches to linear scan
+
+   \param buf - GAP buffer pointer.
+   \param pos - index of the element.
+   \param is_set - output. GAP value (0 or 1).
+   \return GAP index.
+
+    @ingroup SSE4
+*/
+inline
+unsigned sse42_gap_bfind(const unsigned short* BMRESTRICT buf,
+                         unsigned pos, unsigned* BMRESTRICT is_set)
+{
+    unsigned start = 1;
+    unsigned end = 1 + ((*buf) >> 3);
+    unsigned dsize = end - start;
+
+    if (dsize < 17)
+    {
+        start = bm::sse4_gap_find(buf+1, (bm::gap_word_t)pos, dsize);
+        *is_set = ((*buf) & 1) ^ (start & 1);
+        BM_ASSERT(buf[start+1] >= pos);
+        BM_ASSERT(buf[start] < pos || (start==0));
+
+        return start+1;
+    }
+    unsigned arr_end = end;
+    while (start != end)
+    {
+        unsigned curr = (start + end) >> 1;
+        if (buf[curr] < pos)
+            start = curr + 1;
+        else
+            end = curr;
+
+        unsigned size = end - start;
+        if (size < 16)
+        {
+            size += (end != arr_end);
+            unsigned idx =
+                bm::sse4_gap_find(buf + start, (bm::gap_word_t)pos, size);
+            start += idx;
+
+            BM_ASSERT(buf[start] >= pos);
+            BM_ASSERT(buf[start - 1] < pos || (start == 1));
+            break;
+        }
+    }
+
+    *is_set = ((*buf) & 1) ^ ((start-1) & 1);
+    return start;
+}
+
+/**
+    Hybrid binary search, starts as binary, then switches to scan
+    @ingroup SSE4
+*/
+inline
+unsigned sse42_gap_test(const unsigned short* BMRESTRICT buf, unsigned pos)
+{
+    unsigned is_set;
+    bm::sse42_gap_bfind(buf, pos, &is_set);
+    return is_set;
+}
+
+
+
 /**
     Experimental (test) function to do SIMD vector search (lower bound)
     in sorted, growing array
@@ -1281,6 +1637,78 @@ bool sse42_shift_r1_and(__m128i* block,
     return co1;
 }
 
+/**
+    Build partial XOR product of 2 bit-blocks using digest mask
+
+    @param target_block - target := block ^ xor_block
+    @param block - arg1
+    @param xor_block - arg2
+    @param digest - mask for each block wave to XOR (1) or just copy (0)
+
+    @ingroup SSE4
+    @internal
+*/
+inline
+void sse42_bit_block_xor(bm::word_t*  target_block,
+                   const bm::word_t*  block, const bm::word_t*  xor_block,
+                   bm::id64_t digest)
+{
+    for (unsigned i = 0; i < bm::block_waves; ++i)
+    {
+        const bm::id64_t mask = (1ull << i);
+        unsigned off = (i * bm::set_block_digest_wave_size);
+        const __m128i* sub_block = (__m128i*) (block + off);
+        __m128i* t_sub_block = (__m128i*)(target_block + off);
+
+        if (digest & mask) // XOR filtered sub-block
+        {
+            const __m128i* xor_sub_block = (__m128i*) (xor_block + off);
+            __m128i mA, mB, mC, mD;
+            mA = _mm_xor_si128(_mm_load_si128(sub_block),
+                                  _mm_load_si128(xor_sub_block));
+            mB = _mm_xor_si128(_mm_load_si128(sub_block+1),
+                                  _mm_load_si128(xor_sub_block+1));
+            mC = _mm_xor_si128(_mm_load_si128(sub_block+2),
+                                  _mm_load_si128(xor_sub_block+2));
+            mD = _mm_xor_si128(_mm_load_si128(sub_block+3),
+                                  _mm_load_si128(xor_sub_block+3));
+
+            _mm_store_si128(t_sub_block, mA);
+            _mm_store_si128(t_sub_block+1, mB);
+            _mm_store_si128(t_sub_block+2, mC);
+            _mm_store_si128(t_sub_block+3, mD);
+
+            mA = _mm_xor_si128(_mm_load_si128(sub_block+4),
+                                  _mm_load_si128(xor_sub_block+4));
+            mB = _mm_xor_si128(_mm_load_si128(sub_block+5),
+                                  _mm_load_si128(xor_sub_block+5));
+            mC = _mm_xor_si128(_mm_load_si128(sub_block+6),
+                                  _mm_load_si128(xor_sub_block+6));
+            mD = _mm_xor_si128(_mm_load_si128(sub_block+7),
+                                  _mm_load_si128(xor_sub_block+7));
+
+            _mm_store_si128(t_sub_block+4, mA);
+            _mm_store_si128(t_sub_block+5, mB);
+            _mm_store_si128(t_sub_block+6, mC);
+            _mm_store_si128(t_sub_block+7, mD);
+
+        }
+        else // just copy source
+        {
+            _mm_store_si128(t_sub_block , _mm_load_si128(sub_block));
+            _mm_store_si128(t_sub_block+1, _mm_load_si128(sub_block+1));
+            _mm_store_si128(t_sub_block+2, _mm_load_si128(sub_block+2));
+            _mm_store_si128(t_sub_block+3, _mm_load_si128(sub_block+3));
+
+            _mm_store_si128(t_sub_block+4, _mm_load_si128(sub_block+4));
+            _mm_store_si128(t_sub_block+5, _mm_load_si128(sub_block+5));
+            _mm_store_si128(t_sub_block+6, _mm_load_si128(sub_block+6));
+            _mm_store_si128(t_sub_block+7, _mm_load_si128(sub_block+7));
+        }
+    } // for i
+}
+
+
 
 #define VECT_XOR_ARR_2_MASK(dst, src, src_end, mask)\
     sse2_xor_arr_2_mask((__m128i*)(dst), (__m128i*)(src), (__m128i*)(src_end), (bm::word_t)mask)
@@ -1384,9 +1812,28 @@ bool sse42_shift_r1_and(__m128i* block,
 #define VECT_SET_BLOCK_BITS(block, idx, start, stop) \
     sse42_set_block_bits(block, idx, start, stop)
 
-#define VECT_BLOCK_CHANGE(block) \
-    sse42_bit_block_calc_change((__m128i*)block)
+#define VECT_BLOCK_CHANGE(block, size) \
+    sse42_bit_block_calc_change((__m128i*)block, size)
 
+#define VECT_BLOCK_XOR_CHANGE(block, xor_block, size) \
+    sse42_bit_block_calc_xor_change((__m128i*)block, (__m128i*)xor_block, size)
+
+#ifdef BM64_SSE4
+#define VECT_BLOCK_CHANGE_BC(block, gc, bc) \
+    sse42_bit_block_calc_change_bc((__m128i*)block, gc, bc)
+#endif
+
+#define VECT_BIT_FIND_FIRST(src, pos) \
+    sse42_bit_find_first((__m128i*) src, pos)
+
+#define VECT_BIT_FIND_DIFF(src1, src2, pos) \
+    sse42_bit_find_first_diff((__m128i*) src1, (__m128i*) (src2), pos)
+
+#define VECT_BIT_BLOCK_XOR(t, src, src_xor, d) \
+    sse42_bit_block_xor(t, src, src_xor, d)
+
+#define VECT_GAP_BFIND(buf, pos, is_set) \
+    sse42_gap_bfind(buf, pos, is_set)
 
 #ifdef __GNUG__
 #pragma GCC diagnostic pop
