@@ -1,31 +1,33 @@
 #ifndef BMVMIN__H__INCLUDED__
 #define BMVMIN__H__INCLUDED__
 /*
-Copyright(c) 2002-2005 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
+Copyright(c) 2002-2017 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
 
-Permission is hereby granted, free of charge, to any person 
-obtaining a copy of this software and associated documentation 
-files (the "Software"), to deal in the Software without restriction, 
-including without limitation the rights to use, copy, modify, merge, 
-publish, distribute, sublicense, and/or sell copies of the Software, 
-and to permit persons to whom the Software is furnished to do so, 
-subject to the following conditions:
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-The above copyright notice and this permission notice shall be included 
-in all copies or substantial portions of the Software.
+    http://www.apache.org/licenses/LICENSE-2.0
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
-OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
-DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
-ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
-OTHER DEALINGS IN THE SOFTWARE.
-
-For more information please visit:  http://bmagic.sourceforge.net
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
+For more information please visit:  http://bitmagic.io
 */
 
+/*! \file bmvmin.h
+    \brief Mini bitset for testing and utility purposes (internal)
+*/
+
+
+#ifdef _MSC_VER
+#pragma warning( push )
+#pragma warning( disable : 4100)
+#endif
+
 namespace bm
 {
 
@@ -33,10 +35,24 @@ namespace bm
 #define BM_MINISET_GAPLEN (bm::gap_len_table<true>::_len[0])
 #define BM_MINISET_ARRSIZE(x) ((x / 32) + ( (x % 32) && 1 ))
 
-/*! @defgroup mset Small sets functionality
- *  Templates in this group are used to keep block types in BM library.
- *  Classes of this group can tune bvector template (MS parameter)
- *  for best performance or minimal memory usage.
+/*!
+   \brief GAP block to bitblock conversion.
+   \param dest - bitblock buffer pointer.
+   \param buf  - GAP buffer pointer.
+   \param dest_len - length/size of the destination buffer.
+
+    @internal
+*/
+template<typename T>
+void gap_convert_to_bitset(unsigned* dest, const T*  buf,  unsigned  dest_len)
+{
+   ::memset(dest, 0, dest_len * sizeof(unsigned));
+    bm::gap_add_to_bitset(dest, buf);
+}
+
+
+/*! @defgroup mset Small sets functionality (intrenal)
+ *  @internal
  *  @ingroup bmagic
  *  @{
  */
@@ -44,11 +60,9 @@ namespace bm
 
 /*!
     @brief Template class implements memory saving set functionality
-    
-    Template can be used as template parameter for bvector if we 
-    want to tune bvector for minimal memory consumption.
-
+    @ingroup mset
     @sa bvmini
+    @internal
 */
 template <class A, size_t N> class miniset
 {
@@ -135,10 +149,10 @@ public:
     unsigned mem_used() const
     {
         return sizeof(*this) +
-               m_buf ? 
+               (m_buf ?
                  (m_type ? (BM_MINISET_GAPLEN * sizeof(gap_word_t))
                         : (BM_MINISET_ARRSIZE(N) * sizeof(bm::word_t)))
-                : 0; 
+                : 0);
     }
 
     void swap(miniset& mset)
@@ -172,7 +186,7 @@ private:
     void init_gapbuf(bm::word_t* buf)
     {
         unsigned arr_size = 
-            BM_MINISET_GAPLEN / (sizeof(bm::word_t) / sizeof(bm::gap_word_t));
+            unsigned(BM_MINISET_GAPLEN / (sizeof(bm::word_t) / sizeof(bm::gap_word_t)));
         m_buf = A::allocate(arr_size, 0);
         if (buf)
         {
@@ -193,7 +207,7 @@ private:
 
         gap_convert_to_bitset(buf, (gap_word_t*) m_buf, arr_size);
         arr_size = 
-            BM_MINISET_GAPLEN / (sizeof(bm::word_t) / sizeof(bm::gap_word_t));
+            unsigned(BM_MINISET_GAPLEN / (sizeof(bm::word_t) / sizeof(bm::gap_word_t)));
         A::deallocate(m_buf, arr_size);
         m_buf = buf;
         m_type = 0;
@@ -206,18 +220,16 @@ private:
 
 
 /*!
-    @brief Mini bitvector used in bvector template to keep block type flags
-    
-    Template is used as a default template parameter MS for bvector  
-    Offers maximum performance comparing to miniset.
-
+    @brief Mini bit-vector for auxiliary purposes
+    @ingroup mset
     @sa miniset
+    @internal
 */
 template<size_t N> class bvmini
 {
 public:
 
-    bvmini(int start_strategy = 0) 
+    bvmini(int = 0)
     {
         ::memset(m_buf, 0, sizeof(m_buf));
     }
@@ -268,23 +280,31 @@ private:
     @brief Bitvector class with very limited functionality.
 
     Class implements simple bitset and used for internal 
-    and testing purposes. 
+    and testing purposes.
+    @internal
 */
 template<class A> class bvector_mini
 {
 public:
-    bvector_mini(unsigned size) 
+#ifdef BM64ADDR
+    typedef bm::id64_t                                   size_type;
+#else
+    typedef bm::id_t                                     size_type;
+#endif
+
+public:
+    bvector_mini(size_type size)
       : m_buf(0),
         m_size(size)
     {
-        unsigned arr_size = (size / 32) + 1; 
+        size_type arr_size = (size / 32) + 1;
         m_buf = A::allocate(arr_size, 0);
         ::memset(m_buf, 0, arr_size * sizeof(unsigned));
     }
     bvector_mini(const bvector_mini& bvect)
        : m_size(bvect.m_size)
     {
-        unsigned arr_size = (m_size / 32) + 1;
+        size_type arr_size = (m_size / 32) + 1;
         m_buf = A::allocate(arr_size, 0);
         ::memcpy(m_buf, bvect.m_buf, arr_size * sizeof(unsigned));        
     }
@@ -295,41 +315,37 @@ public:
     }
 
     /// Checks if bit pos 1 or 0. Returns 0 if 0 and non zero otherwise.
-    int is_bit_true(unsigned pos) const
+    int is_bit_true(size_type pos) const
     {
         unsigned char  mask = (unsigned char)((char)0x1 << (pos & 7));
-        unsigned char* offs = (unsigned char*)m_buf + (pos >> 3); // m_buf + (pos/8)
-
+        unsigned char* offs = (unsigned char*)m_buf + (pos >> 3);
         return (*offs) & mask;
     }
 
     /// Sets bit number pos to 1
-    void set_bit(unsigned pos)
+    void set_bit(size_type pos)
     {
         unsigned char  mask = (unsigned char)(0x1 << (pos & 7));
         unsigned char* offs = (unsigned char*)m_buf + (pos >> 3); 
         *offs |= mask;
     }
 
-
     /// Sets bit number pos to 0
-    void clear_bit(unsigned pos)
+    void clear_bit(size_type pos)
     {
         unsigned char  mask = (unsigned char)(0x1 << (pos & 7));
-        unsigned char* offs = (unsigned char*)m_buf + (pos >> 3); 
-
-        *offs &= ~mask;
+        unsigned char* offs = (unsigned char*)m_buf + (pos >> 3);
+        *offs &= (unsigned char)~mask;
     }
 
     /// Counts number of bits ON 
-    unsigned bit_count() const
+    size_type bit_count() const
     {
-        register unsigned count = 0;
-        const unsigned* end = m_buf + (m_size / 32)+1;    
-
+        size_type count = 0;
+        const unsigned* end = m_buf + (m_size / 32)+1;
         for (unsigned* start = m_buf; start < end; ++start)
         {
-            register unsigned value = *start;
+            unsigned value = *start;
             for (count += (value!=0); value &= value - 1; ++count);
         }
         return count;
@@ -338,19 +354,16 @@ public:
     /// Comparison.
     int compare(const bvector_mini& bvect)
     {
-        unsigned cnt1 = bit_count();
-        unsigned cnt2 = bvect.bit_count();
-
+        size_type cnt1 = bit_count();
+        size_type cnt2 = bvect.bit_count();
         if (!cnt1 && !cnt2) return 0;
-
-        unsigned cnt_min = cnt1 < cnt2 ? cnt1 : cnt2;
-
+        size_type cnt_min = cnt1 < cnt2 ? cnt1 : cnt2;
         if (!cnt_min) return cnt1 ? 1 : -1;
 
-        unsigned idx1 = get_first();
-        unsigned idx2 = bvect.get_first();
+        size_type idx1 = get_first();
+        size_type idx2 = bvect.get_first();
 
-        for (unsigned i = 0; i < cnt_min; ++i)
+        for (size_type i = 0; i < cnt_min; ++i)
         {
             if (idx1 != idx2)
             {
@@ -359,58 +372,49 @@ public:
             idx1 = get_next(idx1);
             idx2 = bvect.get_next(idx2);
         }
-
-        BM_ASSERT(idx1==0 || idx2==0);
-
         if (idx1 != idx2)
         {
             if (!idx1) return -1;
             if (!idx2) return  1;
             return idx1 < idx2 ? 1 : -1;
         }
-
         return 0;
     }
 
 
     /// Returns index of the first ON bit
-    unsigned get_first() const
+    size_type get_first() const
     {
-        unsigned pos = 0;
+        size_type pos = 0;
         const unsigned char* ptr = (unsigned char*) m_buf;
-
-        for (unsigned i = 0; i < (m_size/8)+1; ++i)
+        for (unsigned i = 0; i < ((m_size/8)+1); ++i)
         {
-            register unsigned char w = ptr[i];
-
-
+            unsigned char w = ptr[i];
             if (w != 0)
             {
-                while ((w & 1) == 0)
+                while ((w & 1u) == 0)
                 {
-                    w >>= 1;
+                    w = (unsigned char)(w >> 1u);
                     ++pos;
                 }
                 return pos;
             }
-            pos += sizeof(unsigned char) * 8;
+            pos = size_type(pos + sizeof(unsigned char) * 8u);
         }
         return 0;
     }
 
 
     /// Returns index of next bit, which is ON
-    unsigned get_next(unsigned idx) const
+    size_type get_next(size_type idx) const
     {
-        register unsigned i;
-
+        size_type i;
         for (i = idx+1; i < m_size; ++i)
         {
             unsigned char* offs = (unsigned char*)m_buf + (i >> 3); 
             if (*offs)
             {
                 unsigned char mask = (unsigned char)((char)0x1 << (i & 7));
-
                 if (*offs & mask)
                 {
                     return i;
@@ -424,13 +428,10 @@ public:
         return 0;
     }
 
-
     void combine_and(const bvector_mini& bvect)
     {
         const unsigned* end = m_buf + (m_size / 32)+1;
-    
         const unsigned* src = bvect.get_buf();
-
         for (unsigned* start = m_buf; start < end; ++start)
         {
             *start &= *src++;
@@ -440,22 +441,17 @@ public:
     void combine_xor(const bvector_mini& bvect)
     {
         const unsigned* end = m_buf + (m_size / 32)+1;
-    
         const unsigned* src = bvect.get_buf();
-
         for (unsigned* start = m_buf; start < end; ++start)
         {
             *start ^= *src++;
         }
     }
 
-
     void combine_or(const bvector_mini& bvect)
     {
         const unsigned* end = m_buf + (m_size / 32)+1;
-    
         const unsigned* src = bvect.get_buf();
-
         for (unsigned* start = m_buf; start < end; ++start)
         {
             *start |= *src++;
@@ -465,9 +461,7 @@ public:
     void combine_sub(const bvector_mini& bvect)
     {
         const unsigned* end = m_buf + (m_size / 32)+1;
-    
         const unsigned* src = bvect.get_buf();
-
         for (unsigned* start = m_buf; start < end; ++start)
         {
             *start &= ~(*src++);
@@ -477,12 +471,11 @@ public:
     const unsigned* get_buf() const { return m_buf; }
     unsigned mem_used() const
     {
-        return sizeof(bvector_mini) + (m_size / 32) + 1;
+        return unsigned(sizeof(bvector_mini) + (m_size / 32) + 1);
     }
 
     void swap(bvector_mini& bvm)
     {
-        BM_ASSERT(m_size == bvm.m_size);
         bm::word_t* buftmp = m_buf;
         m_buf = bvm.m_buf;
         bvm.m_buf = buftmp;
@@ -490,11 +483,15 @@ public:
 
 private:
     bm::word_t*   m_buf;
-    unsigned      m_size;
+    size_type     m_size;
 };
 
 
 
 } // namespace bm
 
+#ifdef _MSC_VER
+#pragma warning( pop )
+#endif
+
 #endif
