@@ -1,24 +1,19 @@
 /*
-Copyright (c) 2002,2003 Anatoliy Kuznetsov.
+Copyright(c) 2002-2017 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
 
-Permission is hereby granted, free of charge, to any person 
-obtaining a copy of this software and associated documentation 
-files (the "Software"), to deal in the Software without restriction, 
-including without limitation the rights to use, copy, modify, merge, 
-publish, distribute, sublicense, and/or sell copies of the Software, 
-and to permit persons to whom the Software is furnished to do so, 
-subject to the following conditions:
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-The above copyright notice and this permission notice shall be included 
-in all copies or substantial portions of the Software.
+    http://www.apache.org/licenses/LICENSE-2.0
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
-OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
-DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
-ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
-OTHER DEALINGS IN THE SOFTWARE.
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+
+For more information please visit:  http://bitmagic.io
 */
 
 #ifndef RLEBTV__H__INCLUDED__
@@ -26,13 +21,14 @@ OTHER DEALINGS IN THE SOFTWARE.
 
 #include <cassert>
 
-/*
+/**
     GAP vector is designed to be very small.
     The size of buffer is fixed 128 short values.
     First word is a start flag.
     Used for debugging purposes.
-*/
 
+    \internal
+*/
 class gap_vector
 {
 public:
@@ -74,7 +70,7 @@ public:
 
     const gap_word_t* get_buf() const { return m_buf; }
 
-    void invert() { *m_buf = 1 - *m_buf; }
+    void invert() { *m_buf = bm::gap_word_t(1 - *m_buf); }
 
     void temp_invert() const;
 
@@ -85,6 +81,8 @@ public:
     gap_word_t* get_buf() { return m_buf; }
 
     int compare(const gap_vector& vect);
+    
+    bool get_last(unsigned* last) const;
 
 private:
     gap_word_t   m_buf[bm::gap_max_buff_len+3];    
@@ -193,12 +191,22 @@ inline void gap_vector::control() const
 
 inline int gap_vector::is_bit_true(unsigned pos) const
 {
-    return bm::gap_test(m_buf, pos);
+    auto r1 = bm::gap_test(m_buf, pos);
+    auto r2 = bm::gap_test_unr(m_buf, pos);
+    assert(r1 == r2);
+    return int(r2);
 }
 
+inline bool gap_vector::get_last(unsigned* last) const
+{
+    bool found = bm::gap_find_last(m_buf, last);
+    return found;
+}
+
+
 inline int gap_vector::test(unsigned pos) const
 {
-    return bm::gap_test(m_buf, pos);
+    return is_bit_true(pos);
 }
 
 
