@@ -34,7 +34,7 @@ void generate_vect_simpl0(VT& vect)
     std::swap(vect, v_tmp);
 }
 
-/// sub-tange vector generator
+/// sub-range vector generator
 ///
 /// @internal
 template<typename VT, typename DISTR, typename MT_RAND>
@@ -117,6 +117,8 @@ void generate_bvector(BV& bv, typename BV::size_type vector_max, bool optimize)
         {
             unsigned len = rand() % 64;
             bv.set_range(i, i + len);
+            bool all_one_range = bv.is_all_one_range(i, i + len);
+            assert(all_one_range);
             i += len;
             if (i > vector_max)
                 break;
@@ -203,6 +205,8 @@ void FillSetsIntervals(BVMINI* bvect_min,
         if (i < end)
         {
             bvect_full.set_range(i, end - 1, set_flag);
+            bool all_one_range = bvect_full.is_all_one_range(i, end - 1);
+            assert(all_one_range == set_flag);
         }
 
         for (j = i; j < end; ++j)
@@ -290,6 +294,8 @@ void FillSetsIntervals(
         if (i < end)
         {
             bvect_full.set_range(i, end - 1, set_flag);
+            bool all_one_range = bvect_full.is_all_one_range(i, end - 1);
+            assert(all_one_range == set_flag);
         }
 
         i = end;
@@ -324,7 +330,7 @@ template<typename SZT>
 SZT random_minmax(SZT min, SZT max)
 {
     SZT r = (unsigned(rand()) << 16u) | unsigned(rand());
-    if (sizeof(SZT) == 8)
+    if (bm::conditional<sizeof(SZT) == 8>::test())
     {
         SZT r2 = (unsigned(rand()) << 16u) | unsigned(rand());
         r |= (r2 << 32);
@@ -798,7 +804,7 @@ void GenerateSV(SV&   sv, unsigned strategy = 0)
         }
         cout << "ok effective size=" << (sv.size() - from) << endl;
     }
-
+    break;
     default:
         break;
     } // switch
@@ -931,3 +937,27 @@ void GenerateCompressedBufferCollection(CBC& cbc)
     } // for
     cbc.sync();
 }
+
+
+template<typename SV>
+void generate_serialization_test_set(SV&   sv,
+                                     typename SV::size_type vector_max)
+{
+    typename SV::back_insert_iterator bi(sv.get_back_inserter());
+
+    unsigned v = 0;
+    for (typename SV::size_type i = 0; i < vector_max; ++i)
+    {
+        unsigned plato = rand() % 16;
+        for (unsigned j = 0; i < vector_max && j < plato; ++i, ++j)
+        {
+            *bi = v;
+        } // for j
+        if (++v > 100000)
+            v = 0;
+        unsigned nulls = rand() % 16;
+        if (nulls)
+            bi.add_null(nulls);
+        i += nulls;
+    } // for i
+}
