@@ -112,6 +112,12 @@ void compare_BV(const BV& bv, const VT& vect, bool compare_count = true)
                      << endl;
                 assert(0); exit(1);
             }
+            bool b = bv.any_range(prev_id, v1);
+            if (r)
+            {
+                assert(b);
+            }
+            (void)b;
         }
         prev_id = v1;
     }
@@ -129,6 +135,215 @@ void compare_BV(const BV& bv, const VT& vect, bool compare_count = true)
 }
 
 
+template<class BV>
+void IntervalsCheck(const BV& bv)
+{
+    BV bv_inv(bv);
+    bv_inv.invert();
+
+    typename BV::size_type intervals = bm::count_intervals(bv);
+    typename BV::size_type intervals_c = 1;
+
+    typename BV::enumerator en1 = bv.get_enumerator(0);
+    typename BV::enumerator en2 = bv_inv.get_enumerator(0);
+
+    while (en1.valid())
+    {
+        typename BV::size_type from = *en1;
+        typename BV::size_type to = *en2;
+        assert(from != to);
+
+        bool all_one, any_one, is_interval;
+        if (to == bm::id_max)
+        {
+            all_one = bv.is_all_one_range(from, to-1);
+            assert(all_one);
+            any_one = bv.any_range(from, to - 1);
+            assert(any_one);
+            break;
+        }
+        else
+        {
+            all_one = bv.is_all_one_range(from, to);
+            assert(!all_one);
+            auto cnt = bv.count_range(from, to);
+            any_one = bv.any_range(from, to);
+            assert(any_one == (cnt>0));
+        }
+
+        if (to == bm::id_max)
+        {}
+        else
+        {
+            ++intervals_c;
+        }
+        if (to < from)
+        {
+            --from;
+            assert(!bv.test(to));
+            all_one = bv.is_all_one_range(from, to);
+            assert(!all_one);
+            typename BV::size_type cnt = bv.count_range(to, from);
+            assert(!cnt);
+            any_one = bv.any_range(to, from);
+            assert(!any_one);
+            is_interval = bm::is_interval(bv, to, from);
+            assert(!is_interval);
+
+            en2.go_to(from+1);
+            if (!en2.valid())
+                break;
+        }
+        else
+        {
+            --to;
+            assert(bv.test(to));
+            all_one = bv.is_all_one_range(from, to);
+            assert(all_one);
+            typename BV::size_type cnt = bv.count_range(from, to);
+            assert(cnt == (to - from + 1));
+            en1.go_to(to+1);
+            any_one = bv.any_range(from, to);
+            assert(any_one);
+            is_interval = bm::is_interval(bv, from, to);
+            assert(is_interval);
+        }
+
+    } // while
+    if (intervals != intervals_c)
+    {
+        typename BV::size_type diff;
+        diff = std::max(intervals, intervals_c) - std::min(intervals, intervals_c);
+        if (diff > 1)
+        {
+            cerr << "Intervals difference:" << diff << endl;
+            assert(0);
+            exit(1);
+        }
+    }
+}
+
+
+template<class BV>
+void interval_copy_range(BV& bv, const BV& bv_src,
+                         typename BV::size_type from, typename BV::size_type to)
+{
+    bv.clear();
+
+    if (from > to)
+        bm::xor_swap(from, to);
+
+    bm::interval_enumerator<BV> ien(bv_src, from, false);
+    while (ien.valid())
+    {
+        auto st = ien.start();
+        assert(st >= from);
+        if (st > to)
+            break;
+        auto end = ien.end();
+        if (end > to)
+            end = to;
+
+        bv.set_range(st, end);
+        if (!ien.advance())
+            break;
+    } // while
+}
+
+template<typename BV>
+void IntervalsEnumeratorCheck(const BV& bv)
+{
+    typename BV::allocator_pool_type pool;
+
+    typename BV::size_type f, l, m;
+    auto b = bv.find_range(f, l);
+    if (!b)
+    {
+        assert(bv.count() == 0);
+        return;
+    }
+    m = l - f;
+    if (!m)
+        m = l;
+
+    bool eq;
+    // Full vector
+    {
+        BV bv2; BV bv2_c;
+        typename BV::mem_pool_guard g1(pool, bv2);
+        typename BV::mem_pool_guard g2(pool, bv2_c);
+
+        bv2_c.copy_range(bv, 0, bm::id_max-1);
+
+        interval_copy_range(bv2, bv, 0, bm::id_max - 1);
+        eq = bv2.equal(bv2_c);
+        assert(eq);
+    }
+    // 0 -> frist
+    {
+        BV bv2, bv2_c;
+        typename BV::mem_pool_guard g1(pool, bv2);
+        typename BV::mem_pool_guard g2(pool, bv2_c);
+
+        bv2_c.copy_range(bv, 0, l);
+
+        interval_copy_range(bv2, bv, 0, l);
+        eq = bv2.equal(bv2_c);
+        assert(eq);
+    }
+    // [first..last]
+    {
+        BV bv2, bv2_c;
+        typename BV::mem_pool_guard g1(pool, bv2);
+        typename BV::mem_pool_guard g2(pool, bv2_c);
+
+        bv2_c.copy_range(bv, f, l);
+
+        interval_copy_range(bv2, bv, f, l);
+        eq = bv2.equal(bv2_c);
+        assert(eq);
+    }
+    // [last..]
+    {
+        BV bv2, bv2_c;
+        typename BV::mem_pool_guard g1(pool, bv2);
+        typename BV::mem_pool_guard g2(pool, bv2_c);
+
+        bv2_c.copy_range(bv, l, bm::id_max-1);
+
+        interval_copy_range(bv2, bv, l, bm::id_max - 1);
+        eq = bv2.equal(bv2_c);
+        assert(eq);
+    }
+    // [mid..last]
+    {
+        BV bv2, bv2_c;
+        typename BV::mem_pool_guard g1(pool, bv2);
+        typename BV::mem_pool_guard g2(pool, bv2_c);
+
+        bv2_c.copy_range(bv, m, l);
+
+        interval_copy_range(bv2, bv, m, l);
+        eq = bv2.equal(bv2_c);
+        assert(eq);
+    }
+    // [first..mid]
+    {
+        BV bv2, bv2_c;
+        typename BV::mem_pool_guard g1(pool, bv2);
+        typename BV::mem_pool_guard g2(pool, bv2_c);
+
+        bv2_c.copy_range(bv, f, m);
+
+        interval_copy_range(bv2, bv, f, m);
+        eq = bv2.equal(bv2_c);
+        assert(eq);
+    }
+
+}
+
+
+
 template<class SV, class Vect>
 bool CompareSparseVector(const SV& sv, const Vect& vect, bool interval_filled = false)
 {
@@ -492,16 +707,25 @@ template<typename CSV>
 void CheckCompressedDecode(const CSV& csv,
     typename CSV::size_type from, typename CSV::size_type size)
 {
-    std::vector<typename CSV::value_type> vect;
+    std::vector<typename CSV::value_type> vect, vect2, vect_tmp;
     vect.resize(size);
+    vect2.resize(size);
+    vect_tmp.resize(size);
 
     typename CSV::size_type sz = csv.decode(&vect[0], from, size);
+    typename CSV::size_type sz2 = csv.decode_buf(&vect2[0], &vect_tmp[0], from, size);
+    assert(sz == sz2);
+
+    typename CSV::const_iterator it = csv.get_const_iterator(from);
     typename CSV::size_type ex_idx = 0;
     for (typename CSV::size_type i = from; i < from + sz; ++i)
     {
         auto v = csv.get(i);
         auto vx = vect[ex_idx];
-        if (v != vx)
+        auto vx2 = vect[ex_idx];
+        auto vx_it = *it;
+
+        if (v != vx || v != vx2 || v != vx_it)
         {
             cerr << "compressed vector decode mismatch from="
                 << from << " idx=" << i
@@ -510,6 +734,7 @@ void CheckCompressedDecode(const CSV& csv,
             assert(0);  exit(1);
         }
         ++ex_idx;
+        ++it;
     }
 }
 
@@ -879,3 +1104,39 @@ void CompareStrSparseVector(const SSV& str_sv,
     } // for
     cout << endl;
 }
+
+
+template<typename BV>
+void TestFindDiff(const BV& bv1, BV& bv2)
+{
+    bool f;
+    typename BV::size_type pos, pos_c, pos_l;
+    f = bv1.find_first_mismatch(bv2, pos);
+    BV bv_x;
+    bv_x.bit_xor(bv1, bv2, BV::opt_compress);
+    if (!f)
+    {
+        auto a = bv_x.any();
+        assert(!a);
+        return;
+    }
+    else // found
+    {
+        bool f2 = bv1.find_first_mismatch(bv2, pos_l, pos);
+        assert(f2 == f);
+        assert(pos_l == pos);
+        if (pos)
+        {
+            f2 = bv1.find_first_mismatch(bv2, pos_l, pos-1);
+            assert(!f2);
+        }
+    }
+
+    bool cf = bv_x.find(pos_c);
+    assert(f == cf);
+    assert(pos == pos_c);
+
+    f = bv2.find_first_mismatch(bv1, pos);
+    assert(f == cf);
+    assert(pos == pos_c);
+}
